/**
 * Program: EventGenerator.java
 * Comments:  This simple program calls a servlet and issues commands in the
 * argument list.  Typically, this program is embedded in a queue script.
 * Based on ideas by Jan Labanowski, et al.
 *
 * The program takes four command line arguments:
 *    a) URL for the receiving program;
 *    b) The user's name;
 *    c) The name of the job;
 *    d) Type of event.
 * The event type can be either "started" or "finished".  All of the 
 * real functionality is contained in the job monitoring servlet.
 */

import java.net.*;
import java.io.*;

public class EventGenerator {

    String monitorURL,eventType,userName;
    public EventGenerator(String args[]) {
	String[] names=new String[3];
	names[0]="user";
	names[1]="job";
	names[2]="event";
	if(checkArgs(args[0],args[3])) 
	    postToServlet(args,names);
    }
    
    private boolean checkArgs(String theURL,String theEvent){
	boolean check=true;
	//A very simple check on the URL.
	if(!theURL.substring(0,4).equals("http")) {
	    System.err.println("Invalid URL supplied");
	    check=false;
	    return check;
	}
	if(!(theEvent.equals("started") || theEvent.equals("finished"))) {
	    System.err.println("Invalid event supplied");
	    check=false;
	    return check;
	}
	return check;	
    }
    
    /** 
     * Uses java's networking package to post to the servlet. Uses
     * java 1.1 compatible methods because this is what most HPC
     * systems have installed.  
     */
    private void postToServlet(String args[],String names[] ){
	String request;
	String response=null;
	java.net.URL url=null;
	java.net.URLConnection conn=null;
	PrintWriter out;
	BufferedReader in=null;
	try {
	    //Construct the request.
	    request=names[0]+"="+args[1];
	    request+="&"+names[1]+"="+args[2];
	    request+="&"+names[2]+"="+args[3];	  
	    
	    System.out.println(args[0]+request);

	    //Open the URL connection
	    url=new java.net.URL(args[0]);
	    conn=url.openConnection();
	    conn.setDoOutput(true);
	    out=new PrintWriter(conn.getOutputStream());
	    out.println(request);
	    out.flush();
	    out.close();
	    
	    //Receive the response.
	    in=new BufferedReader
		(new InputStreamReader(conn.getInputStream()));

	    String line;
	    while((line=in.readLine()) !=null){
		//Do nothing.
	    }
	    in.close();
	}
	catch (Exception e) { 
	    System.err.println("Exception " + e);
	    response = "failure"; 
	} 
    }

    public static void main(String args[]) {
	if(args.length!=4) {
	    System.out.println("Usage: java EventGenerator URL userName jobName eventType");
	    System.exit(1);
	}

	EventGenerator eg=new EventGenerator(args);
    }
}
