/**
 * Program: GOWControl.java
 * Comments: This is the central control program for the client side of
 * GOW.  This is the one you start to run the client.
 * It initializes all of the data and providea accessor methods for 
 * them.  It creates instances of the MyCharon and MyToolBar classes.  A
 * reference of GOWControl must be passed to MyToolBar so that it can 
 * obtain the data.
 * 
 * See the comments for the main method for details on command line 
 * arguments.
 * 
 * This file expects the existence of a file GOW.properties.
 */
package WebFlow.Control;

import java.util.Properties;
import java.net.*;
import java.io.*;
import WebFlow.*;
import WebFlow.event.*;
import WebFlow.Charon.*;
import WebFlow.ToolBar.*;
import org.omg.CORBA.*;

public class GOWControl {

    //Define here all of the data pieces that will need to be
    //made available to contained classes.  
    static URL target;
    String UrlString,userName,descDir,browserCmd,IorUrl,DISPLAY;
    String host,confFile;

    //classes
    //MyCharon mycharon;
    MyToolBar mytoolbar;
    ORB orb;
    WebFlowContext master=null,uc=null;

    /**------------------------------------------------------------
     * The constructor. Note I'm going to pass MyToolBar a 
     * reference to this GOWControl so that it can use the
     * accessor methods.
     *------------------------------------------------------------*/
    public GOWControl(String Port, Properties props) {

	descDir=props.getProperty("descDir");
	browserCmd=props.getProperty("browserCmd");
	UrlString=props.getProperty("UrlString");
	DISPLAY=props.getProperty("DISPLAY");
	host=props.getProperty("host");

	IorUrl=UrlString+"/GOW/IOR/master.txt";

	//Set up the master and slave WebFlow servers.
	String masterIOR=WebFlow.ClientUtils.getIORFromURL(IorUrl);
	orb=WebFlow.ClientUtils.initializeORB(masterIOR,orb);
	master=WebFlow.ClientUtils.getMasterServer(masterIOR,orb);
	userName="gateway";   //change to ClientUtils.getUserName();
	confFile=descDir+"/"+userName+"/"+userName+".conf";
	
	//Start the slave server.  
	System.out.println("Starting slave server");
	//	WebFlow.ClientUtils.createExternalSlave("ssh",host,userName,
	//					DISPLAY, confFile, master);
	System.out.println("Slave server should be running");
	
	//Now add a usercontext to the slave. Note I assume here that the
	//slave server is named after the user, and that the configuration
	//file is setup correctly.
//  	org.omg.CORBA.Object ucobj=
//  	    master.getContext(master.getObjectID()+"/"+userName);
//  	uc=WebFlowContextHelper.narrow(ucobj);
//  	if(uc.getObjectID()==null)
//  	    System.err.println("User context is null");

	//Call MyCharon constructor.
	//mycharon=new MyCharon(this,Port);
	//mycharon.start();

	//Set up the visual display.
	mytoolbar=new MyToolBar(this);
	mytoolbar.pack();
	mytoolbar.show();
    }

    /**------------------------------------------------------------
     * The command line requires two arguments:
     * 1. The port you wish Charon to run over.
     * 2. The full path to the properties file that you want to use.
     *------------------------------------------------------------*/
    public static void main(String[] args) {
	if(args.length < 2) {
	    System.out.println(" ");
	    System.out.println("Usage: java WebFlow.Charon.Client port propFile");
	    System.out.println("where port is the Charon port to use and propFile is");
	    System.out.println("the full path to the properties file.");
	    System.exit(0);
	}
	else {
	    String UrlString="";
	    String Port=args[0];
	    Properties props=new Properties();
	    try {
		props.load(new FileInputStream(args[1]));
		UrlString=props.getProperty("UrlString");
		
	    }
	    catch (Exception ex) {
		System.out.println("Error reading the properties file");
	    }
	    
	    //This is some control code to make sure that all of the
	    //inputs are working OK.
 	    try {
		target = new URL(UrlString);
	    } catch (MalformedURLException e) {
		System.out.println(" ");
		System.out.println("The base url of the target server is incorrect");
		System.out.println(UrlString);
		if(!UrlString.startsWith("http")) {
		    System.out.println("must starts with http");
		}
		System.exit(0);
	    }
	    try {
		InputStream is = target.openStream();
		is.close();
	    } catch (IOException e) {
		System.out.println(" ");
		System.out.println("The target server seems to be down");
		System.out.println(UrlString);
		System.exit(0);
	    }
	    
	    //Everything is fine, so create a new GOWControl object.
	    new GOWControl(Port, props); 
	    
	}
    }

    /**------------------------------------------------------------
     * The following are accessor methods for various pieces of
     * data and classes.  I hope the methods are self explanatory.
     *------------------------------------------------------------*/

    public String getUrlString() { return UrlString; }
    public String getDescDir() { return descDir; }
    public String getUserName() { return userName; }
    public String getBrowserCommand() { return browserCmd; }
    public String getIorUrl() { return IorUrl; }
    public ORB getOrb() { return orb; }
    public WebFlowContext getMaster() { return master; }
    public WebFlowContext getUserContext() { return uc; }
    public String getDisplay() { return DISPLAY; }
    
}
