/**
 * Program Name: parseJob.java
 * Comments: Provides methods for interacting with the job portion of the
 * job description.  A job is defined as a single command to run. 
 * 
 * This program does not handle parsing of the xsil document.  This is done
 * in a separate program such as parseGeneral.
 */

package extensions.generic;

import java.util.*;
import org.escience.XSIL.*;
import java.io.*;

public class parseJob extends XSIL {

    XSIL root;
    Hashtable jobHash=new Hashtable();
    String jobName;

    /**
     * The following two methods are required by the XSIL parent
     * class.  We override them here.  For current purposes there is
     * no distinction between contruct() and instantiate();
     * 
     * Main job of this method is to read all the parameters and
     * store their name/value pairs in a hashtable.
     */ 
    public void construct() {
	for(int ichild=0;ichild<getChildCount(); ichild++) {
	    XSIL x=getChild(ichild);
	    if(x instanceof Param) {
		Param p=(Param)x;
		jobHash.put(p.getName(),p.getText());
	    }
	}
    }
    
    public void instantiate() {
	construct();
    }

    /**
     * Returns the entire hash table of job parameters and values.
     */
    public Hashtable getJobHash() {
	return jobHash;
    }
    

    /**
     * Returns the value for a particular parameter name.
     */
    public String getParamValue(String paramName) {
	if(jobHash.containsKey(paramName))
	    return (String)jobHash.get(paramName);
	else 
	    return null;
    }
    
    /**
     * Returns the list of parameter names as a string array.  
     */
    public String[] getParamNames() {
	String [] names=new String[jobHash.size()];
	Enumeration en=jobHash.keys();
	int i=0;
	while(en.hasMoreElements()) {
	    names[i]=(String)en.nextElement();
	    i++;
	}
	return names;
    }
    

    /**
     * A setter method to add parameter name/value pairs.
     */
    public parseJob setJobParam(String name,String value){
	jobHash.put(name,value);
	return this;
    }

    public void setJobParams(String[] names,String[] values){
	for(int i=0;i<names.length;i++) {
	    jobHash.put(names[i],values[i]);
	}
    }
    
    /**
     * A couple of accessors for the job name.
     */
    public void setJobName(String name) { jobName=name; }
    public String getJobName() { return jobName; }
}


