/**
 * ServerServlet.java
 * This class creates a static client orb and some static methods that 
 * can be accessed by PSE servlets.
 * 
 * @author Tomasz Haupt
 * Modifications by Marlon Pierce.  
 */

package WebFlow;

import WebFlow.*;
import WebFlow.event.*;
import WebFlow.hashtable.*;
import WebFlow.ContextManager.*;
import WebFlow.FB.*;
import WebFlow.RemoteFile.*;
import WebFlow.submitJob.*;
import WebFlow.xml.*;

import org.omg.CORBA.*;
import orbasec.corba.Opaque;
import java.io.*;
import java.util.*;
import java.net.*;

public class ServerServlet {  

    org.omg.CORBA.Object o_base;
    public static ORB orb;
    public static WebFlowContext master;
    String[] args;

    /**
     * These hashtables store usernames and modules as key-value pairs.
     */
    static Hashtable cmRef;
    static Hashtable ucRef;
    static Hashtable fbRef;
    static Hashtable rfRef;
    static Hashtable sjRef;
    static Hashtable aadRef;  


    
    ////////////////////Added///////////////////////
    org.omg.SecurityLevel2.Current current= null;
    ////////////////////Added///////////////////////

    /**
     * Class constructor for ServerServlet.
     * @param none
     * @return void
     * @throws none
     */
    public ServerServlet(String userName){    
	
	//args is a placebo for orb initialization.
	String IorUrl="http://dev-spt01.asc.hpc.mil:8080/SciPortal/IOR/"+userName+".txt";
	args=new String[1];
	args[0]="test";
	cmRef = new Hashtable();
	ucRef = new Hashtable();
	fbRef = new Hashtable();
	sjRef = new Hashtable();
	rfRef = new Hashtable();
	aadRef = new Hashtable();

	initializeORB(IorUrl);
	master=getMasterServer(IorUrl);
	System.out.println("ServerServlet is initialized");
    }
    
    /**
     * Initializes the orb and sets up all of the security parameters.
     * @param IorUrl the string for the master's IOR.
     * @throws SystemException 
     * @throws Exception
     */
    public void initializeORB(String IorURL){
	try {
	    System.out.println("set system properties for ORBacus"); 
	    Properties props = System.getProperties();
	    props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	    props.put("org.omg.CORBA.ORBSingletonClass", "com.ooc.CORBA.ORBSingleton");
	    System.setProperties(props);
	    
	    System.out.println("create ORB");
	    //
	    // Create client ORB 
	    //
	    
	    orbasec.SL2.init(args,props);
	    orb=orbasec.SL2.orb();
	    
	    ///////////Added/////////////////////////////////////////////////
	    // 
	    // Get SecurityLevel2::Current from ORB
	    // 
	    org.omg.SecurityLevel2.Current current = 
		org.omg.SecurityLevel2.CurrentHelper.narrow(
							    orb.resolve_initial_references("SecurityCurrent"));
	    
	    //
	    // Authenticate using PrincipalAuthenticator
	    //
	    org.omg.SecurityLevel2.PrincipalAuthenticator pa;
	    pa = current.principal_authenticator();
	    
	    String mechanism = "Kerberos";
	    int method = 0;
	    
	    byte security_name[] =
		Opaque.encodeKerberosName("gateway/dev-spt01.asc.hpc.mil@tade.asc.hpc.mil").getEncoding();
	    
	    byte auth_data[] =
		(
		 //@@@@ Credentials will not have delegation ability.
		 "delegation=false\n" +
		 "keytab=FILE:/app/gateway/dev-spt01.gateway.keytab\n" +
		 "config=FILE:/app/gateway/orbasec_krb5.conf\n"
		 ).getBytes();
	    
	    org.omg.Security.SecAttribute privileges[] =
		new org.omg.Security.SecAttribute[0];
	    org.omg.SecurityLevel2.CredentialsHolder creds_holder =
		new org.omg.SecurityLevel2.CredentialsHolder();
	    org.omg.Security.OpaqueHolder
		continuation_data =
		new org.omg.Security.OpaqueHolder(),
		auth_specific_data =
		new org.omg.Security.OpaqueHolder();
	    
	    pa.authenticate(
			    method,
			    mechanism,
			    security_name,
			    auth_data,
			    privileges,
			    creds_holder,
			    continuation_data,
			    auth_specific_data
			    );
	    
	    System.out.println("PrincipalAuthenticator ended");
	    
	}
	catch(SystemException ex) {
	    System.err.println(ex.getMessage());
	    ex.printStackTrace();
	    System.exit(1);
	}
	
	catch(Exception ex1) {
	    System.err.println(ex1.getMessage());
	    ex1.printStackTrace();
	    System.exit(1);
	}
    }
    
    public static void setContextManager(String user, ContextManager cm) {
	System.out.println("setContextManager for user: "+user);
	System.out.println("cmID: "+cm.getObjectID()); 
	cmRef.put(user,cm);   
	Enumeration activeUsers = cmRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("setContextManager done"); 
    }
    
    public static ContextManager getContextManager(String user) {
	if(cmRef==null) System.out.println("cmRef is null");
	else System.out.println("cmRef is OK");  
	Enumeration activeUsers = cmRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("This is the line.");
	System.out.println("return ContextManager for "+user);
	return (ContextManager)cmRef.get(user);
    }   

    public static void setUserContext(String user, WebFlowContext uc) {
	System.out.println("setContextManager for user: "+user);
	System.out.println("cmID: "+uc.getObjectID()); 
	ucRef.put(user,uc);   
	/*
	  Enumeration activeUsers = ucRef.keys();
	  while(activeUsers.hasMoreElements()) {
	  System.out.println(activeUsers.nextElement());
	  }
	  System.out.println("setContextManager done");
	*/ 
    }
    
    public static WebFlowContext getUserContext(String user) {
	if(ucRef==null) System.out.println("ucRef is null");
	else System.out.println("ucRef is OK");  
	Enumeration activeUsers = ucRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return ContextManager for "+user);
	return (WebFlowContext)ucRef.get(user);
    }           
    
    public static void setFileBrowser(String user, FBServer fb) {
	fbRef.put(user,fb);   
    }
    
    public static FBServer getFileBrowser(String user) {
	if(fbRef==null) System.out.println("fbRef is null");
	else System.out.println("fbRef is OK");  
	Enumeration activeUsers = fbRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return FileBrowser for "+user);
	return (FBServer)fbRef.get(user);
    }           
    
    public static void setRemoteFile(String user, RemoteFile rf) {
	rfRef.put(user,rf);   
    }
    
    public static RemoteFile getRemoteFile(String user) {
	if(rfRef==null) System.out.println("rfRef is null");
	else System.out.println("rfRef is OK");  
	Enumeration activeUsers = rfRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return RemoteFile for "+user);
	return (RemoteFile)rfRef.get(user);
    }           
    
    public static void setSubmitJob(String user, submitJob sj) {
	sjRef.put(user,sj);   
    }
    
    public static submitJob getSubmitJob(String user) {
	if(sjRef==null) System.out.println("sjRef is null");
	else System.out.println("sjRef is OK");  
	Enumeration activeUsers = sjRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return FileBrowser for "+user);
	return (submitJob)sjRef.get(user);
    }           
    
    public static void setAAD(String user, XmlServer aad) {
	aadRef.put(user,aad);   
    }
    
    public static XmlServer getAAD(String user) {
	if(aadRef==null) System.out.println("aadRef is null");
	else System.out.println("aadRef is OK");  
	Enumeration activeUsers = aadRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return FileBrowser for "+user);
	return (XmlServer)aadRef.get(user);
    }           
    
    
    public WebFlowContext getMasterServer(String IorURL){
	WebFlowContext ws = null;
	try {
	    String masterIOR = getIORFromURL(IorURL);
	    o_base = orb.string_to_object(masterIOR);
	    ws = WebFlowContextHelper.narrow(o_base);
	}
	catch (Exception e) {System.out.println("getMasterServer: "+e);}
	return ws;
    }
    
    public String getIORFromURL(String urlString){
	String masterServerIOR="";
	String thisLine=null;
	try {
	    URL u = new URL(urlString);
	    try {
		DataInputStream theHTML = new DataInputStream(u.openStream());
		try {
		    while ((thisLine = theHTML.readLine()) != null) {
			masterServerIOR += thisLine;
		    } 
		} 
		catch (Exception e) {
		    System.out.println("getIORfromURL:"+e);
		}
	    } 
	    catch (Exception e) {
		System.out.println("getIORfromURL:"+e);
	    }
	    
	} 
	catch (MalformedURLException e) {
	    System.out.println("getIORfromURL:"+urlString + " is not a parseable URL");
	    System.out.println("getIORfromURL:"+e);
	}
	return masterServerIOR;
    }
    

}


