/**
 * CollabManager.java
 * This class creates a static client orb and some static methods that 
 * can be accessed by Collaborative Clients.
 *
 * Written by Choonhan Youn.  
 */

package WebFlow;

import WebFlow.*;
import WebFlow.event.*;
import WebFlow.Collaborator.*;

/*
import WebFlow.hashtable.*;
import WebFlow.ContextManager.*;
import WebFlow.FB.*;
import WebFlow.RemoteFile.*;
import WebFlow.submitJob.*;
import WebFlow.SerialHash.*;
import WebFlow.Charon.*;
*/

import org.omg.CORBA.*;
import java.io.*;
import java.util.*;
import java.net.*;

public class CollabManager implements Runnable {  

    org.omg.CORBA.Object o_base;
    public static ORB orb;
    public static BOA boa;
    public static WebFlowContext master,jspClient;
    String IorUrl;
    String[] args;
    static String propspath = "../Properties/Collab.properties";
    String propsrealpath;
    
    /**
     * These hashtables store usernames and modules as key-value pairs.
     */
    static Hashtable collaboratorRef;
    static Hashtable filenameRef;
    static Hashtable eventRef;
    static Hashtable emailRef;

    static boolean controlflag;
    static String controlUser;
    static String sharingurl;
    
    public CollabManager(String webpath) {
	
	//args is a placebo for orb initialization.
	propsrealpath = webpath+propspath;
	IorUrl="/GOW/IOR/master.txt";
	args=new String[1];
	args[0]="test";

	collaboratorRef = new Hashtable();
	filenameRef = new Hashtable();
	eventRef = new Hashtable();
	emailRef = new Hashtable();
	controlflag = false;
	controlUser = null;
	sharingurl = null;

	//Properties reading
	Properties props = new Properties();
	try {
	    props.load(new FileInputStream(propsrealpath));
	    IorUrl = props.getProperty("UrlString")+IorUrl;
	}
	catch(Exception ex) {
	    System.out.println("Error reading the property file.");
	    System.out.println("property path: "+propsrealpath);
	}

    }
    
    public void run(){
	//Creates the master server.  Also creates an instance of 
	//the charon object and puts it into a static hashtable.
	initializeORB(IorUrl);
	master=getMasterServer(IorUrl);

	/*
	try {
	    jspClient=
		WebFlowContextHelper.narrow(master.addNewContext("JSPClient"));
	}
	catch(Exception e) {
	    System.err.println("Unable to create JSPClient Context");
	    e.printStackTrace();
	}
	*/
	System.out.println("Collaborative Manager is initialized");
    }
    
    /**
     * Initializes the orb and sets up all of the security parameters.
     * @param IorUrl the string for the master's IOR.
     * @throws SystemException 
     * @throws Exception
     */
    public void initializeORB(String IorURL){
	System.out.println("set system properties for ORBacus"); 
	Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass", "com.ooc.CORBA.ORBSingleton");
	System.setProperties(props);
	
	System.out.println("create ORB");
	//
	// Create client ORB 
	//
    
	orb=org.omg.CORBA.ORB.init(args,props);
	boa=((com.ooc.CORBA.ORB)orb).BOA_init(args,props);

	//
	// Set the concurrency model
	//
	com.ooc.CORBA.ORB.conc_model(
	  com.ooc.CORBA.ORB.ConcModel.ConcModelThreaded);
	
	if(com.ooc.CORBA.BOA.conc_model() !=
	   com.ooc.CORBA.BOA.ConcModel.ConcModelThreadPool)
            {
                com.ooc.CORBA.BOA.conc_model(
		 com.ooc.CORBA.BOA.ConcModel.ConcModelThreadPerRequest);
            }
	   
    }

    public static org.omg.CORBA.Object getCollabSession(String username) {

	//Collaborator coll = null;
	org.omg.CORBA.Object p = null;
	if((p = getCollaborator(username)) != null) {
	    return p;
	} else {
	    WebFlowContext wc = createUserContext(master, username);
	    return createUserModule(wc, username);
	}

    }

    public static WebFlowContext createUserContext(WebFlowContext p, String name){
	try {
	    WebFlowContext c = WebFlowContextHelper.narrow(p.addNewContext(name));
	    if(c==null) System.out.println("c is null");
	    return c;
	} catch (Exception e) {
	    System.out.println(e +"while createUserContext");
	    return null;
	}  
    }


    public static org.omg.CORBA.Object createUserModule(WebFlowContext p, String name){
	//Collaborator coll = null;
	org.omg.CORBA.Object p1 = null;
	try {	   
	    p1 = p.addNewModule("Collaborator"); 
	    //coll = CollaboratorHelper.narrow(p1); 

	    if(p1==null) { 
		System.out.println("p1 is null");
		return null;
	    } else {
		System.out.println("Internal HashMap: Adding....");
		setCollaborator(name,p1);
	    }
	} catch (Exception e) {
	    System.out.println(e +"while createUserModule");
	    return null;
	}  

	return p1;
    }
   
    public static void setCollaborator(String user, org.omg.CORBA.Object p) {
	System.out.println("setCollaborator for user: "+user);
	Collaborator coll = CollaboratorHelper.narrow(p);
	System.out.println("collID: "+coll.getObjectID()); 
	collaboratorRef.put(user,p);          
	Enumeration activeUsers = collaboratorRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("setCollaborator done"); 
    }
       
    public static org.omg.CORBA.Object getCollaborator(String user) {
	if(collaboratorRef==null) System.out.println("collaboratorRef is null");
	else System.out.println("collaboratorRef is OK");  
	Enumeration activeUsers = collaboratorRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return Collaborator "+user);
	return (org.omg.CORBA.Object)collaboratorRef.get(user);
    }           

    public static void removeCollaborator(String user) {
	System.out.println("removeCollaborator for user: "+user);

	collaboratorRef.remove(user);   
	Enumeration activeUsers = collaboratorRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("removeCollaborator done"); 
    }

    public static WebFlowContext getContext(String username){
	String masterID = master.getObjectID();
	String usercontextID = masterID+"/"+username;

	WebFlowContext c = null;
	org.omg.CORBA.Object o = master.getContext(usercontextID);
	if(o == null) {
	    System.out.println("cannot find context "+usercontextID);
	}
	c = WebFlowContextHelper.narrow(o);
	return c;
    } 

    public static void removeCollSession(String username) {
	WebFlowContext c = getContext(username);
	try {
	    c.removeMyself();
	}
	catch (Exception e) {;}
	removeCollaborator(username);
    }
    

    public static void eventListeners(String username, String eventID) {

	System.out.println("Event Source ID: "+username);
	System.out.println("Event ID: "+eventID);
	String eachuser = null;

	Enumeration activeUsers = collaboratorRef.keys();
	while(activeUsers.hasMoreElements()) {
	    eachuser = (String)activeUsers.nextElement();
	    System.out.println("Viewer: "+eachuser);

	    master.attachPushEvent(getCollaborator(username), eventID,
				   getCollaborator(eachuser), "getEvent");
	}
    }

    public static void removeEventListeners(String username, String eventID) {

	System.out.println("Remove Event Source ID: "+username);
	System.out.println("Remove Event ID: "+eventID);
	String eachuser = null;

	Enumeration activeUsers = collaboratorRef.keys();
	while(activeUsers.hasMoreElements()) {
	    eachuser = (String)activeUsers.nextElement();
	    System.out.println("Remove Viewer: "+eachuser);

	    master.detachPushEvent(getCollaborator(username), eventID,
				   getCollaborator(eachuser), "getEvent");
	}
    }

    public static Enumeration hashKeys() {
	return collaboratorRef.keys();
    }

    public static void setControlFlag(boolean flag) {
	controlflag = flag;
    }

    public static boolean getControlFlag() {
	return controlflag;
    }

    public static void setControlUser(String name) {
	controlUser = name;
    }

    public static String getControlUser() {
	return controlUser;
    }

    public static void setFileName(String username, String filename) {
	System.out.println("setFileName: "+filename);
	filenameRef.put(username, filename);
    }

    public static String getFileName(String username) {
	String tmp = (String)filenameRef.get(username);
	System.out.println("setFileName: "+tmp);
	return tmp;
    }

    public static void setEmail(String username, String email) {
	System.out.println("setEmail: "+email);
	emailRef.put(username, email);
    }

    public static String getEmail(String username) {
	String tmp = (String)emailRef.get(username);
	System.out.println("setEmail: "+tmp);
	return tmp;
    }

    public static void removeEmail(String username) {
	emailRef.remove(username);
	System.out.println("removeEmail: ");
    }

    public static void setEventName(String username, String filename) {
	System.out.println("setEvent: "+filename);
	eventRef.put(username, filename);
    }

    public static String getEventName(String username) {
	String tmp = (String)eventRef.get(username);
	System.out.println("getEventName: "+tmp);
	return tmp;
    }

    public static void setSharingURL(String urlname) {
	System.out.println("setSharingURL");
	sharingurl = urlname;
    }

    public static String getSharingURL() {
	return sharingurl;
    }

    public WebFlowContext getMasterServer(String IorURL){
	WebFlowContext ws = null;
	try {
	    String masterIOR = getIORFromURL(IorURL);
	    o_base = orb.string_to_object(masterIOR);
	    ws = WebFlowContextHelper.narrow(o_base);
	}
	catch (Exception e) {System.out.println("getMasterServer: "+e);}
	return ws;
    }
    
    public String getIORFromURL(String urlString){
	String masterServerIOR="";
	String thisLine=null;
	try {
	    URL u = new URL(urlString);
	    try {
		DataInputStream theHTML = new DataInputStream(u.openStream());
		try {
		    while ((thisLine = theHTML.readLine()) != null) {
			masterServerIOR += thisLine;
		    } 
		} 
		catch (Exception e) {
		    System.out.println("getIORfromURL:"+e);
		}
	    } 
	    catch (Exception e) {
		System.out.println("getIORfromURL:"+e);
	    }
	    
	} 
	catch (MalformedURLException e) {
	    System.out.println("getIORfromURL:"+urlString + " is not a parseable URL");
	    System.out.println("getIORfromURL:"+e);
	}
	return masterServerIOR;
    }
    
}
