/**
 * ServerServlet.java
 * This class creates a static client orb and some static methods that 
 * can be accessed by PSE servlets.
 * 
 * I'm also going to add a static BOA.
 * I've also noticed this does not have a concurrency model, so I will
 * need to experiment with this. 
 *
 * @author Tomasz Haupt
 * Modifications by Marlon Pierce.  
 */

package WebFlow;

import WebFlow.*;
import WebFlow.event.*;
import WebFlow.hashtable.*;
import WebFlow.ContextManager.*;
import WebFlow.FB.*;
import WebFlow.RemoteFile.*;
import WebFlow.submitJob.*;
import WebFlow.SerialHash.*;
import WebFlow.Charon.*;

import org.omg.CORBA.*;
//import orbasec.corba.Opaque;
import java.io.*;
import java.util.*;
import java.net.*;

public class VizServerServlet implements Runnable {  

    org.omg.CORBA.Object o_base;
    public static ORB orb;
    public static BOA boa;
    public static WebFlowContext master,vizClient;
    public static CharonServer charon;
    public static CharonObserver chobsv;
    String[] args;
    String IorUrl;

    /**
     * These hashtables store usernames and modules as key-value pairs.
     */
    static Hashtable cmRef;
    static Hashtable ucRef;
    static Hashtable fbRef;
    static Hashtable rfRef;
    static Hashtable sjRef;
    static Hashtable aadRef;  
    static Hashtable chRef;
    static Hashtable chobsvRef;
    static Hashtable shRef;
    
    /**
     * Class constructor for ServerServlet.
     * @param none
     * @return void
     * @throws none
     */
    public VizServerServlet(String userName){    
	
	//args is a placebo for orb initialization.
	IorUrl="http://birch.csit.fsu.edu:8001/GOW/IOR/master.txt";
	args=new String[1];
	args[0]="test";
	cmRef = new Hashtable();
	ucRef = new Hashtable();
	fbRef = new Hashtable();
	sjRef = new Hashtable();
	rfRef = new Hashtable();
	aadRef = new Hashtable();
	chRef=new Hashtable();
	chobsvRef=new Hashtable();
	shRef=new Hashtable();
    }
    
    public void run(){
	//Creates the master server.  Also creates an instance of 
	//the charon object and puts it into a static hashtable.
	initializeORB(IorUrl);
	master=getMasterServer(IorUrl);
	try {
	    vizClient=
		WebFlowContextHelper.narrow(master.addNewContext("VizClient"));
	}
	catch(Exception e) {
	    System.err.println("Unable to create VizClient Context");
	    e.printStackTrace();
	}
	System.out.println("ServerServlet is initialized");

//  	charon=CharonServerHelper.narrow(master.addNewModule("Charon"));
//  	charon.test();
//  	chobsv=CharonObserverHelper.narrow(master.addNewModule("CharonObserv"));	chobsv.test();
//  	chobsv.registerObserver(charon);
	
//  	setCharon("master",charon);	
//  	setCharonObsv("master",chobsv);
//  	System.out.println("Charon objects now set");
    }
    
    /**
     * Initializes the orb and sets up all of the security parameters.
     * @param IorUrl the string for the master's IOR.
     * @throws SystemException 
     * @throws Exception
     */
    public void initializeORB(String IorURL){
	    System.out.println("set system properties for ORBacus"); 
	    Properties props = System.getProperties();
	    props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	    props.put("org.omg.CORBA.ORBSingletonClass", "com.ooc.CORBA.ORBSingleton");
	    System.setProperties(props);
	    
	    System.out.println("create ORB");
	    //
	    // Create client ORB 
	    //
	    
	    orb=org.omg.CORBA.ORB.init(args,props);
	    boa=((com.ooc.CORBA.ORB)orb).BOA_init(args,props);

            //
            // Set the concurrency model
            //
            com.ooc.CORBA.ORB.conc_model(
                com.ooc.CORBA.ORB.ConcModel.ConcModelThreaded);

            if(com.ooc.CORBA.BOA.conc_model() !=
               com.ooc.CORBA.BOA.ConcModel.ConcModelThreadPool)
            {
                com.ooc.CORBA.BOA.conc_model(
                    com.ooc.CORBA.BOA.ConcModel.ConcModelThreadPerRequest);
            }

	    
    }
    
    public static void setContextManager(String user, ContextManager cm) {
	System.out.println("setContextManager for user: "+user);
	System.out.println("cmID: "+cm.getObjectID()); 
	cmRef.put(user,cm);   
	Enumeration activeUsers = cmRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("setContextManager done"); 
    }
    
    public static ContextManager getContextManager(String user) {
	if(cmRef==null) System.out.println("cmRef is null");
	else System.out.println("cmRef is OK");  
	Enumeration activeUsers = cmRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return ContextManager for "+user);
	return (ContextManager)cmRef.get(user);
    }   

    public static void setUserContext(String user, WebFlowContext uc) {
	System.out.println("setContextManager for user: "+user);
	System.out.println("cmID: "+uc.getObjectID()); 
	ucRef.put(user,uc);   
	/*
	  Enumeration activeUsers = ucRef.keys();
	  while(activeUsers.hasMoreElements()) {
	  System.out.println(activeUsers.nextElement());
	  }
	  System.out.println("setContextManager done");
	*/ 
    }
    
    public static WebFlowContext getUserContext(String user) {
	if(ucRef==null) System.out.println("ucRef is null");
	else System.out.println("ucRef is OK");  
	Enumeration activeUsers = ucRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return ContextManager for "+user);
	return (WebFlowContext)ucRef.get(user);
    }           
    
    public static void setFileBrowser(String user, FBServer fb) {
	fbRef.put(user,fb);   
    }
    
    public static FBServer getFileBrowser(String user) {
	if(fbRef==null) System.out.println("fbRef is null");
	else System.out.println("fbRef is OK");  
	Enumeration activeUsers = fbRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return FileBrowser for "+user);
	return (FBServer)fbRef.get(user);
    }           
    
    public static void setRemoteFile(String user, RemoteFile rf) {
	rfRef.put(user,rf);   
    }
    
    public static RemoteFile getRemoteFile(String user) {
	if(rfRef==null) System.out.println("rfRef is null");
	else System.out.println("rfRef is OK");  
	Enumeration activeUsers = rfRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return RemoteFile for "+user);
	return (RemoteFile)rfRef.get(user);
    }           
    
    public static void setSubmitJob(String user, submitJob sj) {
	sjRef.put(user,sj);   
    }
    
    public static submitJob getSubmitJob(String user) {
	if(sjRef==null) System.out.println("sjRef is null");
	else System.out.println("sjRef is OK");  
	Enumeration activeUsers = sjRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return submitJob "+user);
	return (submitJob)sjRef.get(user);
    }           

    public static void setSerialHash(String user, SerialHash sh) {
	shRef.put(user,sh);   
    }
    
    public static SerialHash getSerialHash(String user) {
	if(shRef==null) System.out.println("shRef is null");
	else System.out.println("shRef is OK");  
	Enumeration activeUsers = shRef.keys();
	while(activeUsers.hasMoreElements()) {
	    System.out.println(activeUsers.nextElement());
	}
	System.out.println("return SerialHash "+user);
	return (SerialHash)shRef.get(user);
    }           
    
    public static CharonServer getCharon(String master) {
	//I will only add Charon directly to the master, so
	//I don't have to search through the list of usercontexts.
	if(chRef==null) System.out.println("chRef is null");
	else System.out.println("chRef is OK");
	return (CharonServer)chRef.get(master);
    }

    public static void setCharon(String master, CharonServer ch) {
	chRef.put(master,ch);
    }
    
    public static CharonObserver getCharonObsv(String master) {
	//I will only add Charon directly to the master, so
	//I don't have to search through the list of usercontexts.
	if(chRef==null) System.out.println("chRef is null");
	else System.out.println("chobsvRef is OK");
	return (CharonObserver)chobsvRef.get(master);
    }

    public static void setCharonObsv(String master, CharonObserver ch) {
	chobsvRef.put(master,ch);
    }
    
    public WebFlowContext getMasterServer(String IorURL){
	WebFlowContext ws = null;
	try {
	    String masterIOR = getIORFromURL(IorURL);
	    o_base = orb.string_to_object(masterIOR);
	    ws = WebFlowContextHelper.narrow(o_base);
	}
	catch (Exception e) {System.out.println("getMasterServer: "+e);}
	return ws;
    }
    
    public String getIORFromURL(String urlString){
	String masterServerIOR="";
	String thisLine=null;
	try {
	    URL u = new URL(urlString);
	    try {
		DataInputStream theHTML = new DataInputStream(u.openStream());
		try {
		    while ((thisLine = theHTML.readLine()) != null) {
			masterServerIOR += thisLine;
		    } 
		} 
		catch (Exception e) {
		    System.out.println("getIORfromURL:"+e);
		}
	    } 
	    catch (Exception e) {
		System.out.println("getIORfromURL:"+e);
	    }
	    
	} 
	catch (MalformedURLException e) {
	    System.out.println("getIORfromURL:"+urlString + " is not a parseable URL");
	    System.out.println("getIORfromURL:"+e);
	}
	return masterServerIOR;
    }
    
}
