/**
 * Program Name: createVIZ.java
 * Comments: This class is a child of the createScript abstract parent.  
 * Creates a PBS script.
 * 
 * Note that the ContextManager object cm and the string scriptString are
 * protected data inherited from the parent.
 *
 * I am assuming the following naming convention for the context data:
 *
 *   shell: the unix shell to use.
 *   queue: the particular queue (default, challenge, and so on).
 *   Name: the name to give the job.
 *   NNodes: the number of nodes to request.
 *   walltime: the maximum time per node.
 * 
 * Note that all of these variables are not required.
 */

package extensions.generic;
import WebFlow.*;
import WebFlow.ContextManager.*;
import java.io.*;
import java.util.*;

public class createVIZ extends createScript {
    Hashtable ht;

    public createVIZ(ContextManager cm,String userName,String viz){
	super(cm,userName,viz);
    }

    /**
     * This method concatenates the PBS-specific header directives
     * to the string scriptString, inherited from the parent.
     * It probably wouldn't hurt at some point to associate the 
     * param names with their flags (shell and -S, for example).
     */
    public void createHeader() {
        //Set up the working directory
	String comment = 
	    "# Gnuplot script file for plotting data in your data file \n"+
	    "# This script is given the basic function for you. \n"+
	    "# if you need more things or change it, You can modify it. \n"+
	    "# Different Line styles for 2D plot, \n"+
	    "# lines, points, linespoints, impulses, dots, steps, errorbars, \n"+
	    "# boxes, and boxerrorbars \n"+
	    "\n";

        scriptString+=comment;
	checkHash();
    }
    

    public void createFlagTable() {
	ht=new Hashtable();
	ht.put("psmode","set terminal postscript ");
	ht.put("pstitle","set title ");
	ht.put("xlabel","set xlabel ");
	ht.put("ylabel","set ylabel ");

    }

    private void checkHash() {
	String name,val;
	Enumeration en=ht.keys();
	while(en.hasMoreElements()){
	    name=(String)en.nextElement();
	    if(name.equals("psmode")) {
		scriptString+=(String)ht.get(name)
		    +cm.getCurrentSessionProperty(name)
		    +" color "+cm.getCurrentSessionProperty("psls")
		    +" \""+cm.getCurrentSessionProperty("psft")+"\" "
		    +cm.getCurrentSessionProperty("psfs")
		    +"\n";
	    }
	    else if(!(cm.getCurrentSessionProperty(name)==null)){
		scriptString+=(String)ht.get(name)+"\""
		    +cm.getCurrentSessionProperty(name)+"\""+"\n";
	    }    
	}
    }

}





