/**
 * Program Name: parseXMLBean.java
 * Comments: Several convenience methods for manipulating the 
 * ApplDesc.xml file
 * 
 * This Bean may be converted into a webflow module later.
 *
 * Note the package name.  XSIL looks for extensions in a directory called
 * "extensions" in the classpath.  In the ApplDesc.xml file, we have the
 * line
 *       <XSIL Name="Application Descriptor" Type="csm.parseXMLDesc">
 * so we must put parseXMLDesc.class in the extensions/csm/ 
 * directory somewhere in the classpath (such as WEB-INF/classes).
 */

package extensions.generic;

import java.util.*;
import org.escience.XSIL.*;
import java.io.*;

public class parseXMLBean {

    static parseXMLList parsexmllist;
    static XSIL root;
    static parseXMLDesc parsexmldesc;
    static parseXMLHost parsexmlhost;
    static parseXMLViz parsexmlviz;

    public parseXMLBean(){
    }

    public static void init(String xmlFileName) {
	boolean ss = true;
	root = new XSIL(xmlFileName,ss);
	XSIL child;

	for(int ichild=0;ichild<root.getChildCount();ichild++) {
            child = root.getChild(ichild);
	    parsexmllist = (parseXMLList)child;
        }
    }
    
    /**
     * This method should return the names of all of the applications.
     */

    public Vector getApplications() {
	Vector appList = parsexmllist.getParseXMLappListNames();
	return appList;
    }

    /**
     * This method should return the number of input files 
     * for a particular application.
     */
    public String getNumberOfInFiles(String appIdVal) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp = null;

	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}
	tmp = parsexmldesc.getNumberOfInFiles();
	return tmp;
    }

    /**
     * This method should return the value of the parameter
     * for a particular application.
     */
    public String getNumberOfInParams(String appIdVal) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp = null;

	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}
	tmp = parsexmldesc.getNumberOfInParams();
	return tmp;
    }

    public String getNumberOfOutFiles(String appIdVal) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp = null;

	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}
	tmp = parsexmldesc.getNumberOfOutFiles();
	return tmp;
    }

    public String getNumberOfOutParams(String appIdVal) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp = null;

	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}
	tmp = parsexmldesc.getNumberOfOutParams();
	return tmp;
    }

    public String getIOStyle(String appIdVal) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp = null;

	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}
	tmp = parsexmldesc.getIOStyle();
	return tmp;
    }

    /**
     * This method returns the each column of the table for a particular 
     * application. Must provide application id, column name
     */

    public Vector getHostListNames(String appIdVal) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp = null;

	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}

	Vector hostname = parsexmldesc.getHostListNames();
	return hostname;
    }

    public String getMatchTag(String appIdVal, String machineID, String tagID)
     {
	 Vector appList = parsexmllist.getParseXMLappList();
	 Vector appListNames = parsexmllist.getParseXMLappListNames();
	 String tmp1 = null,tmp2=null;
	 String tagVal=null;
	 parseXMLDesc pxml=null;

	 //Find the matching application and host from the lists.
	 for(int j=0;j<appListNames.size();j++) {
	     tmp1=(String)appListNames.elementAt(j);
	     if(tmp1.equals(appIdVal)) {
		 pxml=(parseXMLDesc)appList.elementAt(j);
		 break;
	     }
	 }

	 for(int i = 0; i < pxml.getHostListNames().size(); i++) {
	     tmp2 = (String)pxml.getHostListNames().elementAt(i);
	     if(tmp2.equals(machineID)) {
		 parsexmlhost = (parseXMLHost)pxml.getHostList().elementAt(i);
		 break;
	    }
	}

	//Look for the following specific tags.  We need a better
	//system then this, though.
	if(tagID.equals("QueueType")) 
	    tagVal = parsexmlhost.getQueuetype();
	else if(tagID.equals("ExecPath")) 
	    tagVal = parsexmlhost.getExecpath();
	else if(tagID.equals("WorkDir"))
	    tagVal = parsexmlhost.getWorkdir();
	else if(tagID.equals("QsubPath"))
	    tagVal = parsexmlhost.getQsubpath();
	else if(tagID.equals("CLASSPATH"))
	    tagVal=parsexmlhost.getClasspath();
	else System.err.println("No Table List!!!!");	

	return tagVal;

    }

    public Hashtable getEnvironHash(String appId,String machineId){
	//Find the matching application and host from the lists.
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp1 = null,tmp2=null;
	String tagVal=null;
	parseXMLDesc pxml=null;
	Hashtable returnHash=null;

	for(int j=0;j<appListNames.size();j++) {
	    tmp1=(String)appListNames.elementAt(j);
	    if(tmp1.equals(appId)) {
		System.out.println("Found matching app tag");
		pxml=(parseXMLDesc)appList.elementAt(j);
		break;
	    }
	}
	
	for(int i = 0; i < pxml.getHostListNames().size(); i++) {
	    tmp2 = (String)pxml.getHostListNames().elementAt(i);
	    if(tmp2.equals(machineId)) {
		System.out.println("Found matching machine tag");
		parsexmlhost = (parseXMLHost)pxml.getHostList().elementAt(i);
		break;
	    }
	}
	returnHash=parsexmlhost.getEnvironHash();
	return returnHash;
    }

    public Vector getVizNameList(String appIdVal, String machineID)
    {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListName = parsexmllist.getParseXMLappListNames();
	String tmp = null;
	String tmp1 = null;
	String tmp2 = null;
	Vector tagVals = new Vector();

	for(int i = 0; i < appListName.size(); i++) {
	    tmp = (String)appListName.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}

	for(int i = 0; i < parsexmldesc.getVizList().size(); i++) {
	    parsexmlviz = (parseXMLViz)parsexmldesc.getVizList().elementAt(i);
	    tmp1 = parsexmlviz.getVizHost();
	    
	    if(tmp1.equals(machineID)) {
		tmp2 = parsexmlviz.getVizName();
		tagVals.addElement((String)tmp2); 		    
	    }
	}

	return tagVals;
    }

    public String getMatchTagViz(String appIdVal, String machineID, 
				 String tagID, String tagID1)
    {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListName = parsexmllist.getParseXMLappListNames();
	String tmp = null;
	String tmp1 = null;
	String tmp2 = null;
	String tagVal = null;

	for(int i = 0; i < appListName.size(); i++) {
	    tmp = (String)appListName.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}

	for(int i = 0; i < parsexmldesc.getVizList().size(); i++) {
	    parsexmlviz = (parseXMLViz)parsexmldesc.getVizList().elementAt(i);
	    
	    tmp1 = parsexmlviz.getVizHost();
	    
	    if(tmp1.equals(machineID)) {
		tmp2 = parsexmlviz.getVizName();
		if(tmp2.equals(tagID)) {
		    if(tagID1.equals("VizPath")) {
			tagVal = parsexmlviz.getVizpath();
			break;
		    }
		}
	    }
	}
	return tagVal;
    }


    /* This is for the administrator Page to get information
     */
    public Vector getVtkList(String appIdVal)
    {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListName = parsexmllist.getParseXMLappListNames();
	String tmp = null;
	String tmp1 = null;
	String tmp2 = null;
	Vector tagVals = new Vector();

	for(int i = 0; i < appListName.size(); i++) {
	    tmp = (String)appListName.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}

	for(int i = 0; i < parsexmldesc.getVizList().size(); i++) {
	    parsexmlviz = (parseXMLViz)parsexmldesc.getVizList().elementAt(i);
	    //tmp1 = parsexmlviz.getVizHost();
	    
	    //if(tmp1.equals(machineID)) {
	    tmp2 = parsexmlviz.getVizName();
	    tagVals.addElement((String)tmp2); 		    
	    //}
	}

	return tagVals;
    }

    /* This is for the administrator Page to get information
     */
    public String getVtkHost(String appIdVal, String vizname)
    {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListName = parsexmllist.getParseXMLappListNames();
	String tmp = null;
	String tmp1 = null;
	String tagVals = null;

	for(int i = 0; i < appListName.size(); i++) {
	    tmp = (String)appListName.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}

	for(int i = 0; i < parsexmldesc.getVizList().size(); i++) {
	    parsexmlviz = (parseXMLViz)parsexmldesc.getVizList().elementAt(i);
	    tmp1 = parsexmlviz.getVizName();
	    
	    if(tmp1.equals(vizname)) {
		tagVals = parsexmlviz.getVizHost();
	    }
	}

	return tagVals;
    }

    /* This is for the administrator Page to get information
     */
    public String getVtkPath(String appIdVal, String vizname)
    {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListName = parsexmllist.getParseXMLappListNames();
	String tmp = null;
	String tmp1 = null;
	String tagVals = null;

	for(int i = 0; i < appListName.size(); i++) {
	    tmp = (String)appListName.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}

	for(int i = 0; i < parsexmldesc.getVizList().size(); i++) {
	    parsexmlviz = (parseXMLViz)parsexmldesc.getVizList().elementAt(i);
	    tmp1 = parsexmlviz.getVizName();
	    
	    if(tmp1.equals(vizname)) {
		tagVals = parsexmlviz.getVizpath();
	    }
	}

	return tagVals;
    }

}    





