/**
 * @author Marlon Pierce
 *
 * This file extracts parameter values of the XSIL Host tag.
 */ 
package extensions.generic;
import org.escience.XSIL.*;
import java.util.*;

public class parseXMLHost extends XSIL {

    public String hostname;
    public String queuetype;
    public String execpath;
    public String workdir;
    public String qsubpath;
    public String classpath,ldlibrary,ansysProduct,lmLicense;
    public Hashtable environhash;
    
    boolean instantiated = false;
    
    public void construct(){	
	environhash=new Hashtable();
	for(int ichild=0; ichild < getChildCount(); ichild++){
            XSIL x = getChild(ichild);
            if(x instanceof Param){
		Param p = (Param)x;
		if(p.getName().equals("HostName"))
		    hostname=p.getText();
		if(p.getName().equals("QueueType"))
		    queuetype = p.getText();
		if(p.getName().equals("ExecPath"))
		    execpath = p.getText();
		if(p.getName().equals("WorkDir"))
		    workdir = p.getText();
		if(p.getName().equals("QsubPath"))
		    qsubpath=p.getText();
		if(p.getName().equals("CLASSPATH"))
		    classpath=p.getText();
	    }
	    //The table should be only one row deep.
	    if(x instanceof Table) {
		Table t=(Table)x;
		for(int i=0; i< t.getNColumn();i++) {
		    Column c = t.getColumn(i);
		    String envName=c.getName();
		    for(int j=0; j < t.getNRow(); j++) {
			String envValue = c.getPrimArray().getString(j);
			//A hack because XSIL can't handle a one
			//column, one row table
			if(!envName.trim().equals("EMPTY")) 
			    environhash.put(envName,envValue);
		    }
		}
	    }
	}
    }

    // This function does the heavy lifting of reading arrays and so on
    // It is separate from the construction because it is not called 
    // unless necessary.

    public void instantiate(){
	for(int ichild=0; ichild < getChildCount(); ichild++){
            XSIL x = getChild(ichild);
            if(x instanceof Param){
		Param p = (Param)x;
		if(p.getName().equals("HostName"))
		    hostname=p.getText();
		if(p.getName().equals("QueueType"))
		    queuetype = p.getText();
		if(p.getName().equals("ExecPath"))
		    queuetype = p.getText();
		if(p.getName().equals("WorkDir"))
		    workdir = p.getText();
		if(p.getName().equals("QsubPath"))
		    qsubpath=p.getText();
		if(p.getName().equals("CLASSPATH"))
		    classpath=p.getText();
		if(p.getName().equals("LD_LIBRARY64_PATH"))
		    ldlibrary=p.getText();
		if(p.getName().equals("ANSYS57_PRODUCT"))
		    ansysProduct=p.getText();
		if(p.getName().equals("LM_LICENSE_FILE"))
		    lmLicense=p.getText();
	    }
	}
    }
     
    public String getLMLicense() {
	return lmLicense;
    }
    public String getAnsysProduct() {
	return ansysProduct;
    }
    public String getLDLibrary() {
	return ldlibrary;
    }
    public String getClasspath() {
	return classpath;
    }
    public String getHost() { 
	return hostname; 
    }
    public String getQueuetype() { 
	return queuetype; 
    }
    public String getExecpath() { 
	return execpath; 
    }
    public String getWorkdir() { 
	return workdir; 
    }
    public String getQsubpath() { 
	return qsubpath; 
    }
    public Hashtable getEnvironHash() {
	return environhash;
    }
}
