************************************************************************
C Program: Ising.f
C Purpose: Using Monte Carlo integration to calculate the energy and
C magnetization for the 2D Ising model.  I'm taking J=1.
************************************************************************

      include 'ran2.f'
      implicit none

      integer NITER,NTHERM,NLAT
      parameter(NITER=1500,NTHERM=150)
      parameter(NLAT=30)

      real*8 spins(NLAT,NLAT),int_temp,temp,energy,magnet
      real*8 temp_step
      real*8 sum_energy,sum_magnet,ran2
      real*8 sum2_energy,sum2_magnet,sum4_magnet
      real*8 engArray(NITER),magArray(NITER)
      integer iseed,i,j,k

C Read the input data from standard input.
      
      read(5,*) int_temp,iseed

      
*-----------------------------------------------------------------------
C Loop over several temperatures.
*-----------------------------------------------------------------------
      temp_step=0.25
      do k=1,20
         temp=int_temp+k*temp_step
*-----------------------------------------------------------------------
C     Randomly set spins to start.
*-----------------------------------------------------------------------
         do i=1,NLAT
            do j=1,NLAT
               if(ran2(iseed).lt.0.5) then
                  spins(i,j)=1.
               else
                  spins(i,j)=-1.
               end if
            end do
         end do
         call obsv(spins,NLAT,energy,magnet)
         
*-----------------------------------------------------------------------
C     Thermalize the configuration.
*-----------------------------------------------------------------------
         do i=1,NTHERM
            call monte(spins,temp,iseed,NLAT)
         end do
         
*-----------------------------------------------------------------------
C     Run the calculation.
*-----------------------------------------------------------------------
         do i=1,NITER
            call monte(spins,temp,iseed,NLAT)
            call obsv(spins,NLAT,energy,magnet)
            sum_energy=sum_energy+energy
            sum_magnet=sum_magnet+magnet
            sum2_energy=sum2_energy+energy*energy
            sum2_magnet=sum2_magnet+magnet*magnet
            sum4_magnet=sum4_magnet+magnet*magnet*magnet*magnet
            engArray(i)=energy
            magArray(i)=magnet
         end do
         sum_energy=sum_energy/float(NITER)
         sum_magnet=sum_magnet/float(NITER)
         sum2_magnet=sum2_magnet/float(NITER)
         sum4_magnet=sum4_magnet/float(NITER)
         sum4_magnet=(sum2_magnet-sum_magnet*sum_magnet)/float(NITER)
         
         write(6,*) temp,abs(sum2_magnet)

c     do i=1,NITER
c     write(80,*) engArray(i),magArray(i)
c     write(6,*) engArray(i),magArray(i)
c     end do
      end do
      
 100  continue
      end

************************************************************************
C Subroutine: monte
C Purpose: Sweeps through the lattice, updating spins with Monte Carlo.
************************************************************************
      subroutine monte(spins,temp,iseed,NLAT)
      implicit none
      real*8 spins(NLAT,NLAT)
      real*8 flipped,energy_old,energy_new,temp,sum_nn,delta,ratio,ran2

      integer i,j,im1,jm1,ip1,jp1,iseed,NLAT
     
      do i=1,NLAT
         im1=i-1
         ip1=i+1
         if(im1.le.0) im1=im1+NLAT
         if(ip1.ge.NLAT+1) ip1=ip1-NLAT
         do j=1,NLAT
            jm1=j-1
            jp1=j+1
            if(jm1.le.0) jm1=jm1+NLAT
            if(jp1.ge.NLAT+1) jp1=jp1-NLAT
            flipped=-spins(i,j)
            sum_nn=spins(ip1,j)+spins(im1,j)+spins(i,jp1)+spins(i,jm1)
            energy_new=flipped*sum_nn
            energy_old=spins(i,j)*sum_nn
            delta=(energy_new-energy_old)/temp
            if(delta.ge.0.0) then
               spins(i,j)=flipped
            else
               ratio=exp(delta)
               if(ratio.ge.ran2(iseed)) spins(i,j)=flipped
            end if
         end do
      end do

      return
      end

************************************************************************
C Subroutine obsv
C Purpose: Calculate the energy and the magnetization
************************************************************************
      subroutine obsv(spins,NLAT,energy,magnet)
      implicit none
      
      real*8 spins(NLAT,NLAT)
      real*8 energy,magnet
      integer nlat,i,ip1,j,jp1
      
      magnet=0.0
      energy=0.0
      do i=1,NLAT
         ip1=i+1
         if(ip1.ge.NLAT+1) ip1=ip1-NLAT
         do j=1,NLAT
            jp1=j+1
            if(jp1.ge.NLAT+1) jp1=jp1-NLAT
            magnet=magnet+spins(i,j)
            energy=energy+spins(i,j)*(spins(ip1,j)+spins(i,jp1))
         end do
      end do
      magnet=magnet/float(NLAT*NLAT)
      energy=energy/float(NLAT*NLAT)
      return
      end
      
      
C The following code constitutes the old way that I handled input and
C output.  I'm putting it here so that it's out of the way but I still
C have record of it.

c      character*72 input,output
c      call getarg(iargc(),input)

c      if(iargc().ne.1) then
c         print*,"Usage: a.out input.dat output.dat"
c         goto 100
c      end if

c      input='/npac/home/webflow/WEBFLOW/JIGSAW/Jigsaw/WWW/input.dat'
c      output='/npac/home/webflow/WEBFLOW/JIGSAW/Jigsaw/WWW/data.dat'
      
c      open(10,file=input,status="old")
c      open(80,file=output,status="unknown")

c      read(10,*) temp,iseed

