/**
 * Program: RemoteParseImpl.java
 * Comments: A WebFlow module that writes and retrieves data to and from 
 * XML data in the user's Descriptors directory.
 */

package WebFlow.RemoteParse;

import WebFlow.*;
import java.util.*;
import java.io.*;
import java.text.*;
import extensions.generic.*;

public class RemoteParseImpl extends WebFlow.BeanContextChildSupport
    implements RemoteParseOperations { 
    
    String msg_;
    org.omg.CORBA.Object peer;
    PrintWriter logOut;
    String xmlfilename;
    parseGeneral parsegeneral;

    public RemoteParseImpl(org.omg.CORBA.Object peer, String msg) 
	throws WebFlow.NullPointerException {
	super(peer);
	this.peer=peer;
	String msg_=msg;
	parsegeneral = new parseGeneral();
	xmlfilename = null;

	//Set up logging
	try {
	    if((new File("/tmp/monitor.log").exists())) {
		logOut=
		    new PrintWriter(new FileWriter("/tmp/monitor.log",true));
	    }
	    else {
		logOut=
		    new PrintWriter(new FileWriter("/tmp/monitor.log"));
	    }
	    
	    mylog("====================================================");
	    mylog(getTime());
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	}
    }
    
    public void test() {
	System.out.println("RemoteParse is up");
	mylog("RemoteParse is up");
    }
    
    /**
     * Initialize the object.
     * @param xmlFileName full path name of the JobDesc.xml file
     * @param url the url of the xsil dtd.
     */
    public void init(String xmlfilename, String url) {

	this.xmlfilename = xmlfilename;
	mylog("Job XML is initialized");
	mylog("Opening "+xmlfilename);
	mylog("URL: "+url);	
	parsegeneral.init(xmlfilename,url);
    }

    /**
     * Save the document to a file.
     * @param xmlFileName the name of the file to write xml into.
     */ 
    public void saveDocToFile() {
	mylog("Save the document.");
	parsegeneral.saveDocToFile(xmlfilename);
    }

    /**
     * Getting job Names for the hashtables that the job and task
     * set up.
     */
    public String[] getJobNames() {
	mylog("Getting the job names.");
	Hashtable jobhash = parsegeneral.getJobHash();
	String[] jobNames=new String[jobhash.size()];
	Enumeration keys=jobhash.keys();
	int i=0;
	while(keys.hasMoreElements()) {
	    jobNames[i]=(String)keys.nextElement();
	    i++;
	}
	return jobNames;
    }
	
    public String[] getTaskNames() {
	mylog("Getting the job tasks.");
	Hashtable taskhash = parsegeneral.getTaskHash();
	String[] taskNames=new String[taskhash.size()];
	Enumeration keys=taskhash.keys();
	int i=0;
	while(keys.hasMoreElements()) {
	    taskNames[i]=(String)keys.nextElement();
	    i++;
	}
	return taskNames;
    }

    /**
     * Accessor methods for returning the job's list of parameter
     * names and the value for a particular 
     */	
    public String[] getJobParamNames(String jobname) {
	mylog("Getting the job parametes.");
	return parsegeneral.getJobParamNames(jobname);
    }

    public String getJobParamValue(String jobname, String paramValue) {
	mylog("Getting the value of the job parameter.");
	return parsegeneral.getJobParamValue(jobname,paramValue);
    }

    /**
     * This creates a new parseJob object and adds it to the the
     * hashtable list.
     */
    public void addJob(String jobname) {
	mylog("Creating new parseJob object.");
	parsegeneral.addJob(jobname);
    }

    /**
     * Finds the named job from the hashtable and adds the parameter
     * name and value to it.  If the named parameter already exists, 
     * the value is updated.
     */
    public void addJobParams(String jobname, String paramname, String paramvalue) {
	mylog("Adding the job parameter.");
	parsegeneral.addJobParams(jobname,paramname,paramvalue);
    }

    /**
     * Adds a parseJob to a parent.  Currently the  parent must be the
     * top level document. 
     */
    public void addParseJobNode(String jobname) {
	mylog("Adding a parseJob to a parent.");
	parsegeneral.addParseJobNode(jobname,parsegeneral);
    }

    /**
     * Updates the xml file.  Just deletes the old node and creates
     * a new node with the new values.  Perhaps not the most efficient
     * way of doing this.
     */
    public void updateParseJobNode(String jobname) {
	mylog("Upating the xml file.");
	parsegeneral.updateParseJobNode(jobname,parsegeneral);
    }

    /**
     * Removes the specificed job from the vector list
     */
    public void removeJob(String jobname) {
	mylog("Removing the specified job.");
	parsegeneral.removeJob(jobname);
    }

    /**
     * Removes the specified node from the xml tree.
     */ 
    public void removeParseJobNode(String jobname) {
	mylog("Removing the specified node.");
	parsegeneral.removeParseJobNode(jobname,parsegeneral);
    }
    
    /**
     * Print the argument to the log file.
     */
    private void mylog(String logline) {
	logOut.println(logline);
	logOut.flush();
    }
    
    /**
     * Return the time.
     */
    private String getTime() {
	DateFormat longDate=
	    DateFormat.getDateTimeInstance(DateFormat.FULL,DateFormat.FULL);
	String time=longDate.format(new Date());
	return time;
    }

}






