package extensions.Java3D;

import org.escience.XSIL.*;
import org.escience.Xlook.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;

import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.behaviors.mouse.*;

public class Java3DView extends XSILView {
  Java3D java3d;
  Canvas3D canvas;

  public void instantiate(XSIL x) {
    this.java3d = (Java3D)x;
    this.setLayout(new BorderLayout());
    this.setSize(400,400);

    GraphicsConfiguration config =
           SimpleUniverse.getPreferredConfiguration();
    Canvas3D canvas = new Canvas3D(config);
    this.add(canvas, BorderLayout.CENTER);
    BranchGroup scene = createSceneGraph();
    scene.compile();

    SimpleUniverse simpleU = new SimpleUniverse(canvas);
    simpleU.getViewingPlatform().setNominalViewingTransform();
    simpleU.addBranchGraph(scene);
  }

  public BranchGroup createSceneGraph() {
    Transform3D st = new Transform3D();
    st.setScale(0.5f);
    Transform3D yrot = new Transform3D(); 
    yrot.rotY(-Math.PI/5.0d);
    st.mul(yrot);
    TransformGroup sceneTG = new TransformGroup(st);
    sceneTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    sceneTG.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

    BoundingSphere bounds = new BoundingSphere(new Point3d(0.0,0.0,0.0), 100.0f);
    
    // Add mouse behaviors
    // Create the rotate behavior node
    MouseRotate rotBehavior = new MouseRotate();
    rotBehavior.setTransformGroup(sceneTG);
    sceneTG.addChild(rotBehavior);
    rotBehavior.setSchedulingBounds(bounds);

    // Create the zoom behavior node
    MouseZoom zoomBehavior = new MouseZoom();
    zoomBehavior.setTransformGroup(sceneTG);
    sceneTG.addChild(zoomBehavior);
    zoomBehavior.setSchedulingBounds(bounds);

    // Create the translate behavior node
    MouseTranslate transBehavior = new MouseTranslate();
    transBehavior.setTransformGroup(sceneTG);
    sceneTG.addChild(transBehavior);
    transBehavior.setSchedulingBounds(bounds);

    // sceneTG.addChild(new ColorCube(0.1));
    TransformGroup geomTG = new TransformGroup();
    Java3DGeometry jg;

    Vector geometries = java3d.getGeometries();

    for(int i=0; i<geometries.size(); i++){
      jg = (Java3DGeometry)geometries.elementAt(i);
      int nvertex = jg.getNvertex();
      System.out.println("Processing geometry " + (i+1) + ": vertices = " + nvertex);
      if(nvertex == 0 || jg.getCoordinates() == null){
        System.out.println("Bad Java3DGeometry");
        continue;
      }
      int vertexFormat = GeometryArray.COORDINATES;
      if(jg.getNormals() != null) 
		vertexFormat |= GeometryArray.NORMALS;
      if(jg.getColors() != null) 
		vertexFormat |= GeometryArray.COLOR_3;

      GeometryArray geomarray = null;
      int stripVertexCounts[] = new int[1];
      stripVertexCounts[0] = nvertex;
      String geomtype = jg.getJava3DGeometryType();

      if(geomtype.equals("TriangleArray"))
        geomarray = new TriangleArray(nvertex, vertexFormat);
      else if(geomtype.equals("LineArray"))
        geomarray = new LineArray(nvertex, vertexFormat);
      else if(geomtype.equals("PointArray"))
        geomarray = new PointArray(nvertex, vertexFormat);
      else if(geomtype.equals("QuadArray"))
        geomarray = new QuadArray(nvertex, vertexFormat);
      else if(geomtype.equals("LineStripArray"))
        geomarray = new LineStripArray(nvertex, vertexFormat, stripVertexCounts);
      else if(geomtype.equals("TriangleStripArray"))
        geomarray = new TriangleStripArray(nvertex, vertexFormat, stripVertexCounts);
      else if(geomtype.equals("TriangleFanArray"))
        geomarray = new TriangleFanArray(nvertex, vertexFormat, stripVertexCounts);

      else {
        System.out.println("Java3DGeometryType " + geomtype + " unknown -- ignoring");
        continue;
      }

      geomarray.setCoordinates(0, jg.getCoordinates());
      if(jg.getColors() != null) geomarray.setColors(0, jg.getColors());
      if(jg.getNormals() != null) geomarray.setNormals(0, jg.getNormals());

      Shape3D s = new Shape3D();
      Appearance a = new Appearance();
      PolygonAttributes polatt = new PolygonAttributes();
      polatt.setCullFace(PolygonAttributes.CULL_NONE);
      a.setPolygonAttributes(polatt);
      s.setAppearance(a);

      s.setGeometry(geomarray);
      geomTG.addChild(s);
    }
    sceneTG.addChild(geomTG);

    BranchGroup rootBG = new BranchGroup();
    rootBG.addChild(sceneTG);
    return rootBG;
  }
}
