package extensions.TimeSeries;

import org.escience.XSIL.*;
import org.escience.Xlook.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.applet.*;
import javax.swing.*;
import jclass.chart.*;

public class TimeSeriesView extends XSILView {
  SimpleChart chart = new SimpleChart();
  JButton button = new JButton();
  TimeSeries ts;

  public void instantiate(XSIL x) {
	this.ts = (TimeSeries)x;
	this.setLayout(new BorderLayout());
    this.setSize(400,300);

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(new FlowLayout());
	JCheckBox FFTCheckBox = new JCheckBox("Power Spectrum");
	buttonPanel.add(FFTCheckBox);
//	JButton customizeButton = new JButton("Customize");
//	buttonPanel.add(customizeButton);
	FFTCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) {
			makeGraph(true);
		}
	});

    chart.setHeaderText(ts.getName());

	chart.setAllowUserChanges(true);
	chart.setTrigger(0, 
		new EventTrigger(InputEvent.META_MASK, EventTrigger.CUSTOMIZE));

    JCChartStyle mycs = new  JCChartStyle();
    mycs.setSymbolShape(JCSymbolStyle.NONE);
    mycs.setLineColor(Color.blue);

    chart.getDataView(0).setChartStyle(0,mycs);

	makeGraph(false);
    this.add(chart, BorderLayout.CENTER);
    this.add(buttonPanel, BorderLayout.SOUTH);
  }

  private void makeGraph(boolean useFFT){
    GraphModel gm = new GraphModel(ts, useFFT);
	if(useFFT)
		chart.setYAxisIsLogarithmic(true);
    chart.getDataView(0).setDataSource(gm);
  }
}

class GraphModel implements Chartable {
	TimeSeries ts;
	double t0;
	double dt;
	boolean useFFT;
	FFT f = null;
	int npoints;

  public GraphModel(TimeSeries ts, boolean useFFT) {
    this.ts = ts;
	this.useFFT = useFFT;
	t0 = ts.getStartTime();
	dt = (ts.getEndTime() - ts.getStartTime())/ts.getNdata();

	if(useFFT){
		int power = (int)(0.999999 +
			Math.log((double)ts.getNdata())/Math.log(2.0));
		int nsamppow2 = 1<<power;
		System.out.println(
			"Making FFT: " + ts.getNdata() 
			+ " points rounded to " + nsamppow2);
		f = new FFT(1, nsamppow2, 1);
		double[] data = new double[nsamppow2];
		for(int i=0; i<nsamppow2; i++){
			if(i<ts.getNdata()) data[i] = ts.getData(i);
			else data[i] = 0.0;
		}
		f.calculate(data);
		npoints = nsamppow2/2;
	} else {
		npoints = ts.getNdata();
	}
  }

  public int getDataInterpretation() {
    return Chartable.ARRAY;
  }

  public java.lang.Object getDataItem(int row, int column) {
	
	if(useFFT) {
		if(row == 0) return new Double(column /(dt*npoints));
		if(row == 1) return new Double(f.spectrum(column));
	} else {
		if(row == 0) return new Double(t0 + dt*column);
		if(row == 1) return new Double(ts.getData(column));
	}
    return new Double(-1.0);
  }

  public java.util.Vector getRow(int row) {
    Vector v = new Vector();
    for(int i=0; i<npoints; i++)
      v.addElement(getDataItem(row, i));
    return v;
  }

  public int getNumRows(){
    return 2;
  }
  public String[] getPointLabels() {return null;}
  public String getSeriesName(int row) {return null;}
  public String getSeriesLabel(int row) {return null; }
  public String getName() {return ts.getName();}
}
