package org.escience.Splitter;
import java.io.*;
import com.jclark.xml.parse.*;
import com.jclark.xml.parse.io.*;
import com.jclark.xml.output.*;

/**
 * @version $Revision: 1.7 $ $Date: 1998/05/08 06:42:17 $
 */
public class Splitter {
  public static void main(String args[]) throws IOException {
    if (args.length != 1) {
      System.err.print(
"Usage: java Splitter file.xml\n" +
"The xml file is split into metadata and data files. The metadata is called\n" +
"file_split.xml and the data files file1.dat, file2.dat, etc.\n"
	);
     
      System.exit(1);
    }
    SplitterApp s = new SplitterApp(args[0]);
    s.parse();
  }
}

class SplitterApp extends ApplicationImpl {

  private XMLWriter mainWriter;
  private XMLWriter otherWriter;
  private XMLWriter w;

  private String file;
  private String filebase;
  private String filename;
  private String filesuffix;

  boolean redirect = false;
  int filenum = 0;
  Parser parser;

  public SplitterApp(String file) {
    
    this.file = file;
    int dotindex = file.lastIndexOf(".");
    int q1= file.lastIndexOf('/');
    int q2= file.lastIndexOf('\\');
    int slashindex = (q1>q2 ? q1:q2) +1;
    filebase = file.substring(0, slashindex);
    filename = file.substring(slashindex, dotindex);
    filesuffix = file.substring(dotindex+1);
    System.out.println(filebase +":"+ filename +":"+ filesuffix);

    parser = new ParserImpl();
    parser.setApplication(this);

    String filesplitname = filebase + filename + "_split." + filesuffix;
    try {
      w = this.mainWriter = 
        new UTF8XMLWriter(new FileOutputStream(filesplitname));
    } catch(java.io.FileNotFoundException e) {
      System.out.println("Cannot open " + filesplitname);
      System.exit(1);
    }

      }

  public void parse() {
    try {
      parser.parseDocument(EntityManagerImpl.openFile(file));
    }
    catch (NotWellFormedException e) {
      System.err.println(e.getMessage());
      System.exit(1);
    }
    catch (IOException e) {
      System.out.println("IOException");
      System.exit(1);
    }
  }

  public void startElement(StartElementEvent event) throws IOException {
    boolean redirect = false;
    w.startElement(event.getName());
    int nAtts = event.getAttributeCount();

    for (int i = 0; i < nAtts; i++){
      if(event.getName().equals("Stream") && 
          event.getAttributeName(i).equals("Type") &&
          !event.getAttributeValue(i).equals("Remote")){
        w.attribute(event.getAttributeName(i), "Remote");
        redirect = true;
      } else {
        if(event.getAttributeValue(i).length() > 0){
          w.attribute(event.getAttributeName(i), event.getAttributeValue(i));
        }
      }
    }

    if(redirect) {
      w.flush();
      startRedirect();
    }
  }

  public void endElement(EndElementEvent event) throws IOException {
    endRedirect();
    w.endElement(event.getName());
  }

  public void endDocument() throws IOException {
    w.write('\n');
    w.flush();
  }

  public void processingInstruction(ProcessingInstructionEvent event) throws IOException {
    w.processingInstruction(event.getName(), event.getInstruction());
  }

  public void characterData(CharacterDataEvent event) throws IOException {
    event.writeChars(w);
  }

  public void startRedirect() {
    filenum++;
    try {
      w.write(fileRedirect());
    } catch(java.io.IOException e) {
      System.out.println("Cannot write to MainWriter");
      System.exit(1);
    }

    try {
      otherWriter = new UTF8XMLWriter(new FileOutputStream(filebase + fileRedirect()));
    } catch(java.io.FileNotFoundException e) {
      System.out.println("Cannot open " + fileRedirect());
      System.exit(1);
    }

    w = otherWriter;
  }

  public void endRedirect() {
    if(otherWriter != null){
      try {
        otherWriter.close();
      } catch (java.io.IOException e) {
        e.printStackTrace();
      }
      otherWriter = null;
    }
    w = mainWriter;
  }

  public String fileRedirect() {
    return filename + filenum + ".dat";
  }
}
