package org.escience.XSIL;
import org.escience.XSIL.PrimArray.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;

public class Array extends XSIL {
	int intType;  // one of the MyInputClass types
	int nfound;
	int ndata = 1;
	Stream stream = null;
	int primType = -1;
	Vector dims;
	PrimArray pa = null;

	public void constructor(Element e){
		super.constructor(e);
        	int i=0;
		dims = new Vector();
		for(int idim=0; idim < children.size(); idim++){
			Object o = children.elementAt(idim);
			if(o instanceof Dim){
				dims.addElement((Dim)o);
				ndata *= ((Dim)o).value;
			}
		}

		primType = PrimArray.getTypeFromString(Type);
		if(primType == -1){
			System.out.println("No valid Type for the Array " + this);
		} else {
			stream = getStream();
			if(stream == null)
				System.out.println("No stream found for Array " + this);
		}
	}

      public void instantiate() {
        System.out.println("Instantiating Array " + Name);
        if(stream == null) return;

	  pa = PrimArray.create(primType);
        if(stream.isStreamOfTokens()){
		String tok;
		while((tok = stream.getToken()) != null){
			pa.storeFromString(tok);
		}
        } else {
		DataInputStream dis = stream.getDataInputStream();
		pa.storeFromStream(dis, ndata);
        }
	  nfound = pa.size();
	  if(nfound != ndata)
		System.out.println("ERROR Array expects " + ndata + 
		" elements, but found " + nfound);
      }

	public String dimPrint() {
		String ret = "";
		if(dims != null){
			ret = "";
			for(int i=0; i<dims.size(); i++){
				if(i>0) ret += ",";
				ret += ((Dim)dims.elementAt(i)).value;
			}
		}
		return ret;
	}

	public int getNdim() {
		return dims.size();
	}

	public int getDim(int i) {
		return  ((Dim)dims.elementAt(i)).value;
	}

	public int getNdata() {
		return ndata;
	}

	public PrimArray getPrimArray() {
		if(pa == null) instantiate();
		return pa;
	}

	public String toString() {
		return super.toString() + "[" + dimPrint() + "]";
	}

	public String toLongString(){
		String s = super.toString();
		s += "\tDimensions: [" + dimPrint() + "]\n";
		return s;
	}

	public String print(int howMany){
		String s = super.toString();
		s += "\tDimensions: [" + dimPrint() + "]\n";
		s += "First elements: \n\t";
		if(howMany > pa.size())
			howMany = pa.size();
		String[] paString = pa.getStringArray(0, howMany);
		for(int i=0; i<howMany; i++)
			s += paString[i] + ", ";
		s += "\n";
		return s;
	}

    public void objectToXSILString(
        PrintStream out, Hashtable h, String indent) {

		int k=0;
        out.println(indent + "<Array" + makeNameTypeTags() + ">");

		for(int idim=0; idim< dims.size(); idim++){
			Dim d = (Dim)dims.elementAt(idim);
			out.println(indent + 
				"  <Dim" + d.makeNameTypeTags() + ">"
				+ d.value
				+ "</Dim>");
		}

        out.println(indent + 
			"  <Stream Delimiter=\",\">");
		for(int i=0; i<nfound; i++){
			if(i > 0 && k > 0) out.print(",");
			out.print(pa.getString(i));
			if(k++ > 6){
				k = 0;
				out.print("\n");
			}
		}
		out.println("\n" + indent + 
			"  </Stream>");
		out.println(indent + 
			"</Array>");
    }
}
