package org.escience.XSIL.PrimArray;
import java.util.Vector;
import java.io.*;

class BooleanArray extends PrimArray
{
  boolean defaultObj = false;

  public BooleanArray(int primType, int blockSize){
	super.primType = primType;
	super.data = new Vector();
    super.blockSize = blockSize;
    super.rem = 0;
  }

  public void setDefaultString(String s) throws NumberFormatException{
      defaultObj = (new Boolean(s)).booleanValue();
  }

  public void storeFromString(String s){
    boolean i = defaultObj;
	try {
      i = (new Boolean(s)).booleanValue();
    } catch (NumberFormatException nfe) {
      i = defaultObj;
    }
    store(i);
  }

  public int storeFromStream(DataInput lin, int wanted) {
	int actual = 0;
	try {
      while(actual < wanted){
		store(lin.readBoolean());
		actual++;
	  }
	} catch (IOException e) {}
	return actual;
  }

  public void store(boolean v){
	boolean[] last;
	if(super.data.size() == 0 || super.rem == super.blockSize){
		last = new boolean[super.blockSize];
		super.data.addElement(last);
		super.rem = 0;
    } else {
      last = (boolean[])super.data.lastElement();
	}
	last[rem++] = v;
  }

  public boolean getBoolean(int i){
    int iBlock = i/super.blockSize;
    if(iBlock >= super.data.size()) return defaultObj;
    boolean[] block = (boolean[])super.data.elementAt(iBlock);
    return block[i%super.blockSize];
  }

  public byte[] getByteArray(int index, int lastindex) {
	byte[] ret = new byte[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (byte)(getBoolean(i)?1:0);
    return ret;
  }

  public short[] getShortArray(int index, int lastindex) {
	short[] ret = new short[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (short)(getBoolean(i)?1:0);
    return ret;
  }

  public int[] getIntArray(int index, int lastindex) {
	int[] ret = new int[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (int)(getBoolean(i)?1:0);
    return ret;
  }

  public long[] getLongArray(int index, int lastindex) {
	long[] ret = new long[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (long)(getBoolean(i)?1:0);
    return ret;
  }

  public boolean[] getBooleanArray(int index, int lastindex) {
	boolean[] ret = new boolean[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = getBoolean(i);
    return ret;
  }

  public float[] getFloatArray(int index, int lastindex) {
	int n = lastindex - index;
	float[] ret = new float[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = (float)(getBoolean(i)?1.0:0.0) ;}
    return ret;
  }

  public double[] getDoubleArray(int index, int lastindex) {
    int n = lastindex - index;
	double[] ret = new double[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = (double)(getBoolean(i)?1.0:0.0); }
    return ret;
  }

  public String[] getStringArray(int index, int lastindex) {
    int n = lastindex - index;
    String[] ret = new String[n];
	for(int i=index; i<lastindex; i++){ ret[i-index] = (new Boolean(getBoolean(i))).toString(); }
    return ret;
  }
}
