package org.escience.XSIL.PrimArray;
import java.util.Vector;
import java.util.StringTokenizer;
import java.io.*;

class DoubleComplexArray extends PrimArray
{
  double defaultDoubleReal = (double)0.0;
  double defaultDoubleImag = (double)0.0;

  public DoubleComplexArray(int primType, int blockSize){
	super.primType = primType;
	super.data = new Vector();
    super.blockSize = blockSize;
    super.rem = 0;
  }

  public void setDefaultString(String s) throws NumberFormatException{
	StringTokenizer st = new StringTokenizer(s, ", ");
	if(st.hasMoreTokens()) 
	  defaultDoubleReal = (new Double(st.nextToken())).doubleValue();
	if(st.hasMoreTokens()) 
	  defaultDoubleImag = (new Double(st.nextToken())).doubleValue();
  }

  public void storeFromString(String s){
	double re = defaultDoubleReal;
	double im = defaultDoubleImag;
	StringTokenizer st = new StringTokenizer(s, ", ");
	try {
	  if(st.hasMoreTokens()) 
	    re = (new Double(st.nextToken())).doubleValue();
	  if(st.hasMoreTokens()) 
	    im = (new Double(st.nextToken())).doubleValue();
	} catch (NumberFormatException nfe) {
	}

    store(re, im);
  }

  public int storeFromStream(DataInput lin, int wanted) {
	int actual = 0;
	try {
      while(actual < wanted){
		store(lin.readDouble(), lin.readDouble());
		actual++;
	  }
	} catch (IOException e) {}
	return actual;
  }

  public void store(double re, double im){
	double[] last;
	if(super.data.size() == 0 || super.rem == super.blockSize){
	  last = new double[2*super.blockSize];
	  super.data.addElement(last);
	  super.rem = 0;
    } else {
      last = (double[])super.data.lastElement();
	}
	last[2*rem] = re;
	last[2*rem+1] = im;
    rem++;
  }

  public double getDouble(int i){
    int iBlock = i/super.blockSize;
    if(iBlock >= super.data.size()) return defaultDoubleReal;
    double[] block = (double[])super.data.elementAt(iBlock);
    return block[2*(i%super.blockSize)];
  }
  public double getDoubleImag(int i){
    int iBlock = i/super.blockSize;
    if(iBlock >= super.data.size()) return defaultDoubleImag;
    double[] block = (double[])super.data.elementAt(iBlock);
    return block[2*(i%super.blockSize)+1];
  }

  public byte[] getByteArray(int index, int lastindex) {
	byte[] ret = new byte[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (byte)getDouble(i);
    return ret;
  }

  public short[] getShortArray(int index, int lastindex) {
	short[] ret = new short[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (short)getDouble(i);
    return ret;
  }

  public int[] getIntArray(int index, int lastindex) {
	int[] ret = new int[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (int)getDouble(i);
    return ret;
  }

  public long[] getLongArray(int index, int lastindex) {
	long[] ret = new long[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (long)getDouble(i);
    return ret;
  }

  public boolean[] getBooleanArray(int index, int lastindex) {
	boolean[] ret = new boolean[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] 
      = getDouble(i)==0.0?false:true;
    return ret;
  }

  public float[] getFloatArray(int index, int lastindex) {
	int n = lastindex - index;
	float[] ret = new float[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = (float)getDouble(i) ;}
    return ret;
  }

  public float[] getFloatImagArray(int index, int lastindex) {
	int n = lastindex - index;
	float[] ret = new float[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = (float)getDoubleImag(i) ;}
    return ret;
  }

  public double[] getDoubleArray(int index, int lastindex) {
    int n = lastindex - index;
	double[] ret = new double[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = getDouble(i); }
    return ret;
  }

  public double[] getDoubleImagArray(int index, int lastindex) {
    int n = lastindex - index;
	double[] ret = new double[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = getDoubleImag(i); }
    return ret;
  }

  public String[] getStringArray(int index, int lastindex) {
    int n = lastindex - index;
    String[] ret = new String[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = 
	  Double.toString(getDouble(i)) + "," + Double.toString(getDoubleImag(i)); } 
    return ret;
  }
}
