package org.escience.XSIL.PrimArray;
import java.util.Vector;
import java.io.*;

class FloatArray extends PrimArray
{
  float defaultObj = 0;

  public FloatArray(int primType, int blockSize){
	super.primType = primType;
	super.data = new Vector();
    super.blockSize = blockSize;
    super.rem = 0;
  }

  public void setDefaultString(String s) throws NumberFormatException{
      defaultObj = (new Float(s)).floatValue();
  }

  public void storeFromString(String s){
    float i = defaultObj;
	try {
      i = (new Float(s)).floatValue();
    } catch (NumberFormatException nfe) {
      i = defaultObj;
    }
    store(i);
  }

  public int storeFromStream(DataInput lin, int wanted) {
	int actual = 0;
	try {
      while(actual < wanted){
		store(lin.readFloat());
		actual++;
	  }
	} catch (IOException e) {}
	return actual;
  }

  public void store(float v){
	float[] last;
	if(super.data.size() == 0 || super.rem == super.blockSize){
		last = new float[super.blockSize];
		super.data.addElement(last);
		super.rem = 0;
    } else {
      last = (float[])super.data.lastElement();
	}
	last[rem++] = v;
  }

  public float getFloat(int i){
    int iBlock = i/super.blockSize;
    if(iBlock >= super.data.size()) return defaultObj;
    float[] block = (float[])super.data.elementAt(iBlock);
    return block[i%super.blockSize];
  }

  public byte[] getByteArray(int index, int lastindex) {
	byte[] ret = new byte[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (byte)getFloat(i);
    return ret;
  }

  public short[] getShortArray(int index, int lastindex) {
	short[] ret = new short[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (short)getFloat(i);
    return ret;
  }

  public int[] getIntArray(int index, int lastindex) {
	int[] ret = new int[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (int)getFloat(i);
    return ret;
  }

  public long[] getLongArray(int index, int lastindex) {
	long[] ret = new long[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] = (long)getFloat(i);
    return ret;
  }

  public boolean[] getBooleanArray(int index, int lastindex) {
	boolean[] ret = new boolean[lastindex - index];
	for(int i=index; i<lastindex; i++) ret[i-index] 
      = getFloat(i)==(float)0.0?false:true;
    return ret;
  }

  public float[] getFloatArray(int index, int lastindex) {
	int n = lastindex - index;
	float[] ret = new float[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = (float)getFloat(i) ;}
    return ret;
  }

  public double[] getDoubleArray(int index, int lastindex) {
    int n = lastindex - index;
	double[] ret = new double[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = (double)getFloat(i); }
    return ret;
  }

  public String[] getStringArray(int index, int lastindex) {
    int n = lastindex - index;
    String[] ret = new String[n];
	for(int i=index; i<lastindex; i++) { ret[i-index] = Float.toString(getFloat(i)); }
    return ret;
  }
}
