package org.escience.XSIL.PrimArray;
import java.io.*;

/* What I really want to do here is to extend DataInputStream, but all
the methods there are final. So I have reimplemented the ones I want to,
but just used variable din, whihc is the DataInputStream, used its
methods explicitly for the others */

public class LittleEndianDecoder extends FilterInputStream implements DataInput {
  DataInputStream din;

  public LittleEndianDecoder(InputStream in) {
    super(in);
	din = new DataInputStream(in);
  }

/* These are the methods I don't want to implement */
  public void readFully(byte[] b) throws IOException 
	{din.readFully(b);}
  public void readFully(byte[] b, int i, int j) throws IOException
	{din.readFully(b, i, j);}
  public int skipBytes(int i) throws IOException
	{return din.skipBytes(i);}
  public String readLine() throws IOException {
//    return din.readLine();
    return 
	"readLine is in interface DataInput, but deprecated in DataInputStream!";
  }
  public String readUTF() throws IOException
	{return din.readUTF();}

  public boolean readBoolean() throws IOException {
    int bool = in.read();
    if (bool == -1) throw new EOFException();
    return (bool != 0);
  }

  public byte readByte() throws IOException {
    int temp = in.read();
    if (temp == -1) throw new EOFException();
    return (byte) temp;
  }

  public int readUnsignedByte() throws IOException {
    int temp = in.read();
    if (temp == -1) throw new EOFException();
    return temp;
  }

  public short readShort() throws IOException {
    int byte1 = in.read();
    int byte2 = in.read();
    // only need to test last byte read
    // if byte1 is -1 so is byte2
    if (byte2 == -1) throw new EOFException();
    return (short) ((byte2 << 8) + byte1);
  }

  public int readUnsignedShort() throws IOException {
    int byte1 = in.read();
    int byte2 = in.read();
    if (byte2 == -1) throw new EOFException();
    return (byte2 << 8) + byte1;
  }

  public char readChar() throws IOException {
    int byte1 = in.read();
    int byte2 = in.read();
    if (byte2 == -1) throw new EOFException();
    return (char) ((byte2 << 8) + byte1);
  }

  public int readInt() throws IOException {
    int byte1, byte2, byte3, byte4;
    synchronized (this) {
      byte1 = in.read();
      byte2 = in.read();
      byte3 = in.read();
      byte4 = in.read();
    }
    if (byte4 == -1) {
      throw new EOFException();
    }
    return (byte4 << 24) + (byte3 << 16) + (byte2 << 8) + byte1;
  }

  public long readLong() throws IOException {
    long byte1 = in.read();
    long byte2 = in.read();
    long byte3 = in.read();
    long byte4 = in.read();
    long byte5 = in.read();
    long byte6 = in.read();
    long byte7 = in.read();
    long byte8 = in.read();
    if (byte8 == -1) {
      throw new EOFException();
    }
    return 
	  (byte8 << 56) + (byte7 << 48) + (byte6 << 40) + (byte5 << 32) +
      (byte4 << 24) + (byte3 << 16) + (byte2 << 8) + byte1;
  }

  public final double readDouble() throws IOException {
    return Double.longBitsToDouble(this.readLong());
  }
  
  public final float readFloat() throws IOException {
    return Float.intBitsToFloat(this.readInt());
  }
}
