package org.escience.XSIL.PrimArray;
import java.util.Vector;
import java.io.*;
import java.net.*;

public class PrimArray
{
  Vector data;
  int rem;
  int blockSize;
  int primType;
  static boolean debug = true;

  public final static int BYTE = 1;
  public final static int SHORT = 2;
  public final static int INT = 3;
  public final static int LONG = 4;
  public final static int BOOLEAN = 5;
  public final static int FLOAT = 6;
  public final static int DOUBLE = 7;
  public final static int FLOATCOMPLEX = 8;
  public final static int DOUBLECOMPLEX = 9;
  public final static int STRING = 10;

  public static int getTypeFromString(String type){
    String LCType = type.toLowerCase().trim();

    if(LCType.equals("byte"))return BYTE;

    if(LCType.equals("short"))return SHORT;
    if(LCType.equals("int_2u"))return SHORT;

    if(LCType.equals("int"))return INT;
    if(LCType.equals("int_4u"))return INT;
    if(LCType.equals("int_4s"))return INT;

    if(LCType.equals("long"))return LONG;

    if(LCType.equals("boolean"))return BOOLEAN;

    if(LCType.equals("float"))return FLOAT;
    if(LCType.equals("real_4"))return FLOAT;

    if(LCType.equals("double"))return DOUBLE;
    if(LCType.equals("real_8"))return DOUBLE;

    if(LCType.equals("floatcomplex"))return FLOATCOMPLEX;
    if(LCType.equals("complex_4"))return FLOATCOMPLEX;

    if(LCType.equals("doublecomplex"))return DOUBLECOMPLEX;
    if(LCType.equals("complex_8"))return DOUBLECOMPLEX;
    if(LCType.equals("complex"))return DOUBLECOMPLEX;

    if(LCType.equals("string"))return STRING;
    if(LCType.equals("char"))return STRING;
    if(LCType.equals("character"))return STRING;
    if(LCType.equals("lstring"))return STRING;
    return -1;
  }

  int BIG_INT = 1000000000;

// creating
  public static PrimArray create(int primType){ 
    return create(primType, 1000); 
  }

  public static PrimArray create(int primType, int blockSize){
  switch(primType){
    case BYTE: return new ByteArray(primType, blockSize);
    case SHORT: return new ShortArray(primType, blockSize);
    case INT: return new IntArray(primType, blockSize);
    case LONG: return new LongArray(primType, blockSize);
    case BOOLEAN: return new BooleanArray(primType, blockSize);
    case FLOAT: return new FloatArray(primType, blockSize);
    case DOUBLE: return new DoubleArray(primType, blockSize);
    case FLOATCOMPLEX: return new FloatComplexArray(primType, blockSize);
    case DOUBLECOMPLEX: return new DoubleComplexArray(primType, blockSize);
    case STRING: return new StringArray(primType, blockSize);
    default: return null;
    }
  }

// Utility
  public int size() {
    int blks = data.size();
    if(blks == 0) return 0;
    return blockSize*(blks-1) + rem;
  }

  public int getType() {return primType;}

// String input
  public void storeFromString(String s){}
  public void setDefaultString(String s){}

// DataInput
  public int storeFromStream(DataInput di) {
    return storeFromStream(di, BIG_INT);
  }
  public int storeFromStream(DataInput di, int wanted) {return 0;}

// output methods
  public byte getByte(int index)
    {byte[] b = getByteArray(index, index+1); return b[0];}
  public byte[] getByteArray(int index, int lastindex) {return null;}

  public short getShort(int index)
    {short[] b = getShortArray(index, index+1); return b[0];}
  public short[] getShortArray(int index, int lastindex) {return null;}

  public int getInt(int index)
    {int[] b = getIntArray(index, index+1); return b[0];}
  public int[] getIntArray(int index, int lastindex) {return null;}

  public long getLong(int index)
    {long[] b = getLongArray(index, index+1); return b[0];}
  public long[] getLongArray(int index, int lastindex) {return null;}

  public boolean getBoolean(int index)
    {boolean[] b = getBooleanArray(index, index+1); return b[0];}
  public boolean[] getBooleanArray(int index, int lastindex) {return null;}

  public float getFloat(int index)
    {float[] b = getFloatArray(index, index+1); return b[0];}
  public float[] getFloatArray(int index, int lastindex){return null;}

  public double getDouble(int index)
    {double[] b = getDoubleArray(index, index+1); return b[0];}
  public double[] getDoubleArray(int index, int lastindex){return null;}

  public float getFloatImag(int index)
    {float[] b = getFloatImagArray(index, index+1); return b[0];}
  public float[] getFloatImagArray(int index, int lastindex){
// usually the imaginary part is zero. this is overridden only
// in the complex classes
    int n = lastindex - index;
    float[] b = new float[n];
    for(int i=0; i<n; i++) b[i] = (float)0.0;
    return b;
  }

  public double getDoubleImag(int index)
  {double[] b = getDoubleImagArray(index, index+1); return b[0];}
  public double[] getDoubleImagArray(int index, int lastindex){
// usually the imaginary part is zero. this is overridden only
// in the complex classes
    int n = lastindex - index;
    double[] b = new double[n];
    for(int i=0; i<n; i++) b[i] = 0.0;
    return b;
  }

  public String getString(int index)
    {String[] b = getStringArray(index, index+1); return b[0];}
  public String[] getStringArray(int index, int lastindex){return null;}

// *********** File, Base64, LittleEndian utility classes

// This method is used by all those below
  public int storeFromInputStream(
    InputStream is, 
    int wanted, 
    boolean isBase64, 
    boolean isLittleEndian){

if(debug) try{System.out.println(
"Storing from Stream, wanted= " + wanted + " available=" + is.available());}
catch (IOException e){}

    InputStream is2 = is;
    if(isBase64) is2 = new Base64Decoder(is);

    DataInputStream dis;
    if(isLittleEndian) 
      return storeFromStream(new LittleEndianDecoder(is2), wanted);
    else 
      return storeFromStream(new DataInputStream(is2), wanted);
  }

// from a file
  public int storeFromFile(
      String filename, boolean isBase64, boolean isLittleEndian) 
      throws FileNotFoundException {
    return storeFromFile(filename, BIG_INT, isBase64, isLittleEndian);
  }
  public int storeFromFile(
      String filename, int wanted, boolean isBase64, boolean isLittleEndian) 
      throws FileNotFoundException {

if(debug) System.out.println(
"Storing from File, name=" + filename + " wanted=" + wanted);

    return storeFromInputStream(
      new BufferedInputStream(
      new FileInputStream(filename)),
      wanted, isBase64, isLittleEndian);
  }

// from a URL
  public int storeFromURL(
      String urlname, 
      boolean isBase64, 
      boolean isLittleEndian) throws MalformedURLException, IOException {
    return storeFromURL(urlname, BIG_INT, isBase64, isLittleEndian);
  }
  public int storeFromURL(
      String urlname, 
      int wanted, 
      boolean isBase64, 
      boolean isLittleEndian) throws MalformedURLException, IOException {

if(debug) System.out.println(
"Storing from URL, name=" + urlname + " wanted=" + wanted);

    URL url = new URL(urlname);
    return storeFromInputStream(
      new BufferedInputStream(url.openStream()),
      wanted, isBase64, isLittleEndian);
  }

// from a string
  public int storeFromBase64String(
      String data, 
      boolean isLittleEndian) {
    return storeFromBase64String(data, BIG_INT, isLittleEndian);
  }
  public int storeFromBase64String(
      String data, 
      int wanted, 
      boolean isLittleEndian) {

if(debug) System.out.println(
"Storing from Base64 String, wanted=" + wanted);

    StringInputStream sis = new StringInputStream(data);
    return storeFromInputStream(
      sis,
      wanted, true, isLittleEndian);
  }
}
