package org.escience.XSIL;
import org.escience.XSIL.PrimArray.*;
import java.util.*;
import java.io.*;
import org.w3c.dom.*;

public class Table extends XSIL {
	int ncolumn;
	int nrow = 0;
	Vector columns;
	boolean noStrings = true;
	Stream stream;
	boolean instantiated = false;

	public void constructor(Element e){
		super.constructor(e);

		columns = new Vector();
		Object o;
		for(int i=0; i<children.size(); i++){
			o = children.elementAt(i);
			if(o instanceof Column) {
				Column c = (Column)o;
				columns.addElement(c);
				if(c.getPrimArray().getType() == PrimArray.STRING)
					noStrings = false;
			}
		}

		ncolumn = columns.size();
		stream = getStream();
		if(stream == null)
			System.out.println("No stream found for Table " + this);
	}

	public void instantiate(){
		int ntoken = 0;
		Column c;
		System.out.println("Instantiating Table " + Name);

		if(stream == null){
			System.out.println("No data for Table " + this);
			nrow = 0;
			return;
		}
			
        	if(stream.isStreamOfTokens()){
			String tok;
			while((tok = stream.getToken()) != null){
				c = (Column)columns.elementAt(ntoken%ncolumn);
				c.getPrimArray().storeFromString(tok);
				ntoken++;
			}
        	} else {
			DataInputStream dis = stream.getDataInputStream();
			try{
			while(dis.available() > 0){
				c = (Column)columns.elementAt(ntoken%ncolumn);
				if(c.getPrimArray().storeFromStream(dis, 1) ==0)
					break;
				ntoken++;
			}
			} catch(IOException e){}
        	}

		nrow = ntoken/ncolumn;
		if(debug) System.out.println(
			"Table has " + ncolumn + " columns and " + nrow + " rows");

		instantiated = true;
	}

	public int getNColumn() {
		return columns.size();
	}

	public int getNRow() {
		if(!instantiated) instantiate();
		return nrow;
	}

	public Column getColumn(int i) {
		if(!instantiated) instantiate();
		return (Column)columns.elementAt(i);
	}

	public String toString() {
		String ret = super.toString();
		ret += " [" + ncolumn + " columns, " + nrow + " rows]";
		return ret;
	}

	public String toLongString(){
		String s = toString();
		Column c;
		for(int i=0; i<columns.size(); i++){
			c = (Column)columns.elementAt(i);
			s += c.toLongString();
		}
		return s;
	}

    public void objectToXSILString(
        PrintStream out, Hashtable h, String indent) {

        out.println(indent + "<Table" + makeNameTypeTags() + ">");

        for(int icol=0; icol< ncolumn; icol++){
            Column c = (Column)columns.elementAt(icol);
            out.println(indent +
                "  <Column" + c.makeNameTypeTags() + "/>");
		}

        out.println(indent +
            "  <Stream Delimiter=\",\">");
        for(int irow=0; irow<nrow; irow++){
			for(int icol=0; icol<ncolumn; icol++){
				Column c = (Column)columns.elementAt(icol);
				if(icol > 0) out.print(",");
				out.print(c.getPrimArray().getString(irow));
            }
        }
        out.println("\n" + indent +
            "  </Stream>");
        out.println(indent +
            "</Table>");
    }
}
