package org.escience.Xlook;
import org.escience.XSIL.*;
import org.escience.XSIL.PrimArray.*;
import java.io.*;

public class Matlab {

	DataOutputStream dout;
	PrintStream md;
	int ObjectNumber;  // to give generic names to objects, eg "Array3"

	public static void main(String args[]) {
		String xmlfile = null;
		String datafile = null;
		String mdatafile = null;
		if(args.length > 2) {
			xmlfile = args[0];
			datafile = args[1];
			mdatafile = args[2];
		} else {
			System.out.println("Usage: Test file.xml file.data file.m");
			System.exit(1);
		}

		XSIL root = new XSIL(xmlfile, false);

		if(root == null){
			System.out.println("No XSIL element found.");
			System.exit(1);
		}

		Matlab m = new Matlab(root, datafile, mdatafile);
	}

	public Matlab(XSIL root, String datafile, String mdatafile){
		FileOutputStream out;
		try {
			out = new FileOutputStream(datafile);
			dout = new DataOutputStream(out);
		} catch(Exception e) {
			System.out.println("Cannot open " + datafile);
			System.exit(1);
		}

		try {
			out = new FileOutputStream(mdatafile);
			md = new PrintStream(out);
		} catch(Exception e) {
			System.out.println("Cannot open " + mdatafile);
		}

		ObjectNumber = 0;
		md.println("fid = fopen('" + datafile + "');");
		printRecurse(root);
		md.println("fclose(fid);");
		md.println("who");
		try {
			md.close();
			dout.close();
		} catch(Exception e) {
			System.out.println("Cannot close!");
		}
	}

	public void printRecurse(XSIL x) {
        XSIL child;
		WriteXsil(x);

		if(x.hasChildNodes()){
			for(int i=0; i<x.getChildCount(); i++){
				child = x.getChild(i);
				printRecurse(child);
			}
		}
	}

	public void WriteXsil(XSIL x) {
		String Name = x.getName();
		if(Name == null || Name.length() == 0) 
			Name = x.getClasstag() + ObjectNumber++;
		else
			Name = cleanUp(Name);

		if(x instanceof Param){
			Param p = (Param)x;
			if(p.getType().toLowerCase().equals("string")){
				md.println(Name + " = '" + p.getText() + "'");
			} else {
				md.println(Name + " = " + p.getText());
			}
		}
		if(x instanceof Array){
			Array a = (Array)x;

// write metadata
			md.print(Name + " = fread(fid, " + a.getNdata() + ", '");
			switch(a.getPrimArray().getType()){
				case PrimArray.SHORT:  md.print("int16");  break;
				case PrimArray.INT:    md.print("int32");  break;
				case PrimArray.LONG:   md.print("int64");  break;
				case PrimArray.FLOAT:  md.print("float32"); break;
				case PrimArray.DOUBLE: md.print("double"); break;
			}
			md.print( "');\n");

// write binary data
			try {
				for(int i=0; i<a.getNdata(); i++){
					dout.writeDouble(a.getPrimArray().getDouble(i));
				}
			} catch(IOException e) {
				System.err.println("Error writing to DataOutputStream");
			}
		}
	}

	public String cleanUp(String s) {
		String ss = s;
		ss = ss.replace(':', '_');
		return ss;
	}
}
