package org.escience.Xlook;
import org.escience.XSIL.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

public class Xlook extends JFrame implements ActionListener {

	XSILAdapter xTreeModel = null;
	XSIL xSelect;

    static void main(String[] argv) {
        
        JFrame frame = null;
        if (argv.length < 1) {
            frame = new Xlook();
        } else {
            frame = new Xlook(title+": "+argv[0], argv[0]);
        }
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
         public void windowClosing(java.awt.event.WindowEvent e) {
             System.exit(0);
         }
        });
        frame.setSize(790, 790);
        frame.setVisible(true);
    }

    public static final boolean DEBUG = false;

    static final String title = "Xlooker";
    static final String openString = "Open";
    static final String viewString = "View Selection";
    static final String printXSILString = "Save Selection As XSIL...";
    static final String printMatlabString = "Save Selection As Matlab...";
    static final String quitString = "Quit";
    static final String reloadString = "Reload current XML file";
    static final String expandString = "Expand Tree";
    static final String collapseString = "Collapse Tree";

    String fname;
    JTree m_tree;
    JTextArea sourceText, messageText;
    Vector textLine;
    OpenDialog openDialog = null;
    SaveAsDialog saveAsDialog = null;

    public Xlook() {
        this(title, null);
    }

    public Xlook(String title, String filename) {
        super(title);
        fname = filename;
        JMenuItem item;
        JMenuBar jmb = new JMenuBar();

        JMenu fileMenu = new JMenu("File");
            item = new JMenuItem(openString);
            fileMenu.add(item);
            item.addActionListener(this);

            item = new JMenuItem(printXSILString);
            fileMenu.add(item);
            item.addActionListener(this);

            item = new JMenuItem(printMatlabString);
            fileMenu.add(item);
            item.addActionListener(this);

			item = new JMenuItem(quitString);
			fileMenu.add(item);
			item.addActionListener(this);

        JMenu viewMenu = new JMenu("View");
            item = new JMenuItem(viewString);
            viewMenu.add(item);
            item.addActionListener(this);

        JMenu shortcutMenu = new JMenu("Shortcuts");
            item = new JMenuItem(expandString);
            shortcutMenu.add(item);
            item.addActionListener(this);

			item = new JMenuItem(collapseString);
			shortcutMenu.add(item);
			item.addActionListener(this);

			item = new JMenuItem(reloadString);
			shortcutMenu.add(item);
			item.addActionListener(this);

        jmb.add(fileMenu);
        jmb.add(viewMenu);
        jmb.add(shortcutMenu);
        setJMenuBar(jmb);

        getContentPane().add(createUI(fname));
    }

    JComponent createUI(String filename) {
        if (DEBUG) System.out.println("START createUI:"+filename);
 
        // create the message panel first so we can send messages to it...
        messageText = new JTextArea(3,40);
        messageText.setFont(new Font("Helvetica", Font.PLAIN, 12));
        JPanel messagePanel = new JPanel(new BorderLayout());
        messagePanel.add(new JScrollPane(messageText) {
            public Dimension getPreferredSize(){ 
                Dimension size = Xlook.this.getSize();
                return new Dimension(size.width, size.height / 4); 
                }
            public Dimension getMinimumSize(){ 
                return new Dimension(100, 100); 
                }
            }, 
            BorderLayout.CENTER);
        messagePanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("Messages"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
       
        // create the TextArea for XML source
        sourceText = new JTextArea();
        sourceText.setFont(new Font("Courier", Font.PLAIN, 10));
        sourceText.setBackground(Color.white);
        sourceText.setForeground(Color.black);
        sourceText.setSelectedTextColor(Color.black);
        sourceText.setSelectionColor(Color.red);
        //sourceText.setOpaque(true);
        sourceText.setEditable(false);
        //sourceText.setEnabled(true);
        JPanel sourcePanel = new JPanel(new BorderLayout());
        sourcePanel.add(new JScrollPane(sourceText){
            public Dimension getPreferredSize(){ 
                Dimension size = Xlook.this.getSize();
                return new Dimension(size.width / 2, size.height * 3 / 5); 
                }
            public Dimension getMinimumSize(){ 
                return new Dimension(100, 100); 
                }
            }, 
            BorderLayout.CENTER);
        sourcePanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("Source View"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
       
        // create the JTree and scroll pane.
        JPanel treePanel = new JPanel(new BorderLayout());
        createJTree();
        treePanel.add(new JScrollPane(m_tree) {
            public Dimension getPreferredSize(){ 
                Dimension size = Xlook.this.getSize();
                return new Dimension(size.width / 2, size.height * 3 / 5); 
                }
            public Dimension getMinimumSize(){ 
                return new Dimension(100, 100); 
                }
            }, 
            BorderLayout.CENTER);

        treePanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("Tree View"),
            BorderFactory.createEmptyBorder(4, 4, 4, 4)
            ));
          
        // refreshUI loads everthything!    
        refreshUI(filename);
        
        // use the new JSplitPane to dynamically resize...
        JComponent split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, 
            true, treePanel, sourcePanel);

        JComponent mainSplitPane = 
            new JSplitPane(JSplitPane.VERTICAL_SPLIT, 
                           true, split, messagePanel);

        if (DEBUG) System.out.println("END createUI:"+filename);
        return mainSplitPane;
    }

    /** refreshUI is called when we have a new filename to parse.
     */
    void refreshUI(String filename) {
        if (DEBUG) System.out.println("START refreshUI:"+filename);
try{
messageText.replaceRange("", 0, messageText.getLineEndOffset(messageText.getLineCount()-1)-1);
}catch(javax.swing.text.BadLocationException e){};
        
        if (filename == null || filename.equals("")) {
            messageText.setForeground(Color.red);
            messageText.append("No input XML filename specified:"+filename+"\n");
            return;           
        }

        try {
			File file = new File(filename);
            FileInputStream fis = new FileInputStream(file);
        } catch (FileNotFoundException fnf) {
            messageText.setForeground(Color.red);
            messageText.append("Cannot open"+filename+"\n");
			System.out.println("Cannot open"+filename);
            return;           
		}
        
        fname = filename;
        // new JTree Model
        parseFile(filename);
        
        // new Source 
        readXMLFile(fname, sourceText);
        sourceText.select(0, 0 );

        setTitle(title+": "+filename);
 
        if (m_tree!= null)
            expandTree();
            
        if (DEBUG) System.out.println("END refreshUI:"+filename);
		nodeSelected(xTreeModel.root, false);
    }

    public void parseFile(String filename) {
        if (DEBUG) System.out.println("START parseFile:"+filename);

		ErrorEater ee = new ErrorEater(messageText);

		xTreeModel = new XSILAdapter();

		xTreeModel.fill(filename, ee);
        m_tree.setModel(xTreeModel);

		m_tree.setShowsRootHandles(true);
		m_tree.collapseRow(0);
		m_tree.setCellRenderer(new XMLTreeCellRenderer());
		m_tree.getSelectionModel().setSelectionMode
			(TreeSelectionModel.SINGLE_TREE_SELECTION);

// Listen for when the selection changes, call nodeSelected(node)
// Lets use a MouseListener instead of a TreeSelectionListener, that way
// we can detect a double click.

        m_tree.addTreeSelectionListener(
            new TreeSelectionListener() {
                public void valueChanged(TreeSelectionEvent e) {
					if(DEBUG) System.out.println("TreeSelectionEvent: " + e);
                    Object node = e.getPath().getLastPathComponent();
                    nodeSelected(node, false);
                }
            }
        );
/*
		m_tree.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				JTree tree = (JTree)e.getSource();
				Object node = tree.getLastSelectedPathComponent();
				boolean doubleclick = (e.getClickCount() > 1);
				nodeSelected(node, doubleclick);
			}
			public void mousePressed(MouseEvent e) { }
			public void mouseReleased(MouseEvent e) { }
			public void mouseEntered(MouseEvent e) {}
			public void mouseExited(MouseEvent e) {}
		});
*/
	}
    synchronized void readXMLFile(String filename, JTextArea ta) {

        if (DEBUG) System.out.println("START readXMLFile"+filename);
        if (filename == null || filename.equals(""))
            return;
        File file = new File(filename);
        FileInputStream fis = null;
        BufferedReader dis = null;

        try {
            fis = new FileInputStream(file);
            dis = new BufferedReader(new InputStreamReader(fis));
        } catch (FileNotFoundException fnf) {
            System.out.println("ERROR: XML4J.readXMLFile: "+fnf);
            return;
        }

        String line;
        int i = 0;

        int len = 0;
        textLine = new Vector();
        String nl = "\n";
        int nllen = nl.length();
        StringBuffer sb = new StringBuffer();

        try{
            while ((line = dis.readLine()) != null) {
                sb.append(line+nl);
                //ta.append(line+nl);
                textLine.addElement(new Integer(len));
                len += line.length()+nllen;
            }
            textLine.addElement(new Integer(len));
        } catch (IOException io) {
            System.out.println(io);
            return;
        }
try{
ta.replaceRange("", 0, ta.getLineEndOffset(ta.getLineCount()-1)-1);
}catch(javax.swing.text.BadLocationException e){};
ta.insert(sb.toString(), 0);
       // ta.revalidate();

        if (DEBUG) System.out.println("END readXMLFile"+filename);
        return;
    }

    void createJTree() {
        m_tree = new JTree();
        if (m_tree == null){
			System.out.println("Null Tree!!");
            return; // should never happen.
		}

        m_tree.setRowHeight(24);
        m_tree.setFont(new Font("Helvetica", Font.PLAIN, 12));
    }

    public void actionPerformed(java.awt.event.ActionEvent e) {
        if (DEBUG) System.out.println("ACTION: "+e.getActionCommand()+", "+e.paramString());
  
        if (e.getActionCommand().equals(quitString)) {
          System.exit(0);
        }
        else if (e.getActionCommand().equals(openString)) {

			if(openDialog == null) openDialog = new OpenDialog();
			refreshUI(openDialog.runit()); 
        }
        else if (e.getActionCommand().equals(expandString)) {
            expandTree();
        }
        else if (e.getActionCommand().equals(collapseString)) {
            int rows = m_tree.getRowCount();
            for (int i = 0; i < rows; i++) {
                m_tree.collapseRow(i);
            }
        }
        else if (e.getActionCommand().equals(reloadString)) {
            refreshUI(fname);
        }
        else if (e.getActionCommand().equals(viewString)) {
			Viewer v = new Viewer(xSelect);
			v.setVisible(true);
        }
        else if (e.getActionCommand().equals(printXSILString)) {
			Hashtable h = new Hashtable();
			if(saveAsDialog == null) saveAsDialog = new SaveAsDialog();
			String filename = saveAsDialog.runit();
			System.out.println("Save to " + filename);
			try {
				File file = new File(filename);
				FileOutputStream fos = new FileOutputStream(file);
				PrintStream ps = new PrintStream(fos);
				xSelect.toXSILString(ps, h);
				ps.close();
				
			} catch (IOException fnf) {
				System.out.println("Cannot open "+filename);
			}
		}
        else if (e.getActionCommand().equals(printMatlabString)) {
			if(saveAsDialog == null) saveAsDialog = new SaveAsDialog();
			String filename = saveAsDialog.runit();
			System.out.println("Save to " + filename);
			Matlab m = new Matlab(xSelect, filename, filename+".mat");
		}
    }

    void expandTree() {
        int rows = 0;
        for (int levels=0; levels <= 4; levels++) {
            rows=m_tree.getRowCount();
            for (int i = 0; i < rows; i++) {
                m_tree.expandRow(i);
            }
        }
    }

    class XMLTreeCellRenderer extends DefaultTreeCellRenderer
    {
        public Component getTreeCellRendererComponent(
			JTree tree, Object value, boolean selected, boolean expanded,
			boolean leaf, int row, boolean hasFocus) {

            Component comp = super.getTreeCellRendererComponent(
				tree, value, selected,  expanded, leaf,  row, hasFocus);

            if (selected)
                comp.setBackground(Color.blue);

            if (value != null && value instanceof XSIL) {
                XSIL x = (XSIL)value;
				setIcon(new ImageIcon("icon/" + x.getClasstag() + ".gif"));
            }

            return comp;
        }
    }

	public void nodeSelected(Object node, boolean doubleclick) {
		if(node instanceof XSIL){
			xSelect = (XSIL)node;
			if(doubleclick) {
				Viewer v = new Viewer(xSelect);
				v.setVisible(true);
			} else {
				messageText.append(xSelect.toLongString());
			}
		} else {
			 messageText.append("Strange Object: " + node);
		}
	}

    class OpenDialog extends JFrame {
		JFileChooser jfc;
		String file = "";
        public OpenDialog() {
            super("Open File");
			String[] xml = new String[] {"xml"};
			jfc = new JFileChooser(".");
			jfc.setMultiSelectionEnabled(false);
			jfc.addChoosableFileFilter(new SimpleFileFilter
				(xml, "XML files (*.xml)"));
			this.getContentPane().add(jfc);
		}

		public String runit() {
			int option = jfc.showOpenDialog(this);
			if(option == JFileChooser.APPROVE_OPTION){
				file = jfc.getSelectedFile().getAbsolutePath();
				System.out.println("OpenDialog: file " + file);
			} else {
				file = "null";
			}
			return file;
		}
	}

    class SaveAsDialog extends JFrame {
		JFileChooser jfc;
		String file = "";
        public SaveAsDialog() {
            super("Save Selection As ");
			jfc = new JFileChooser(".");
			this.getContentPane().add(jfc);
		}

		public String runit() {
			int option = jfc.showSaveDialog(this);
			if(option == JFileChooser.APPROVE_OPTION){
				file = jfc.getSelectedFile().getAbsolutePath();
				System.out.println("SaveAsDialog: file " + file);
			} else {
				file = "null";
			}
			return file;
		}
	}
}
