/*
 * @(#)sysmacros_md.h	1.42 98/09/15
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#ifndef _JAVASOFT_SOLARIS_SYSMACROS_MD_H_
#define _JAVASOFT_SOLARIS_SYSMACROS_MD_H_

#include <stdlib.h> /* for malloc, free, ... */
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>

#define INT_OP(x,op,y)  ((x) op (y))
#define NAN_CHECK(l,r,x) x
#define IS_NAN(x) isnan(x)

#ifdef DEBUG
void panic (const char *, ...);
#define sysAssert(expression) {		\
    if (!(expression)) {		\
	panic("\"%s\", line %d: assertion failure\n", __FILE__, __LINE__); \
    }					\
}
#else
#define sysAssert(expression) ((void) 0)
#endif

/*
 * Solaris always runs in Total Store Order mode for Sparc and
 * Intel doesn't seem to support a weak consistency model for MP
 * so we define this to do nothing.
 */
#define sysStoreBarrier() ((void) 0)

/*
 * Flush the write buffer
 */

#ifdef NATIVE
#ifdef FOR_N32
#define sysMemoryFlush() __synchronize()
#else /* ifndef FOR_N32 */
void sysMemoryFlush(); /* defined in mflush_mips.s */
#endif /* ifndef FOR_N32 */
#else /* ifndef NATIVE */
#define sysMemoryFlush()
#endif /* ifndef NATIVE */

#ifdef FOR_N32
long *sysInvokeNative(size_t , size_t, int, 
		      void *, size_t *, int, char *, size_t *);
#else /* ifndef FOR_N32 */
long *sysInvokeNative(int, size_t, size_t, size_t, size_t,
		      void *, size_t *, int, char *, size_t *,
		      unsigned char *);
#endif

/* name of the default JIT compiler to be used on Solaris */
#define DEFAULT_JIT_NAME "mipsjit"

#define JVM_ONLOAD_SYMBOLS   {"JVM_OnLoad"}

#endif /* !_JAVASOFT_SOLARIS_SYSMACROS_MD_H_ */
