/**
 * Program: submitJobImpl.java
 * Comments:
 * Several utility methods for communicating with the backend.  This is done
 * locally, or with either rsh/rcp or ssh/scp.  The particular flavor of 
 * remote execution is specified in the command line.
 */
package WebFlow.submitJob;

import WebFlow.*;
import org.omg.CORBA.*;
import java.io.*;
import java.util.*;
import java.text.*;

public class submitJobImpl extends WebFlow.BeanContextChildSupport
  implements submitJobOperations {
  
  String msg_;
  org.omg.CORBA.Object peer;
  PrintWriter logFile;
    
    public submitJobImpl(org.omg.CORBA.Object peer, String msg) throws WebFlow.NullPointerException {
	
	super(peer);
	this.peer=peer;
	String msg_=msg;
	
	//Set up logging
        try {
	  if((new File("/tmp/monitor.log").exists())) {
                logFile=
		  new PrintWriter(new FileWriter("/tmp/submitjob.log",true));
	  }
	  else {
	    logFile=
	      new PrintWriter(new FileWriter("/tmp/submitjob.log"));
	  }
	  
	  mylog("====================================================");
	  mylog(getTime());
        }
        catch(Exception ex) {
	  ex.printStackTrace();
        }
    }

    public void test() {
	System.out.println("SubmitJob is up");
    }
    
  /**============================================================
   * Execute the command on the same machine as the server 
   * containing the submitJob module.  Returns the standard
   * output of the command, if any.  Debugging messages are 
   * written to /tmp/monitor.log.
   *============================================================*/
    public String execLocalCommand(String command) {
	Runtime rt = Runtime.getRuntime();
	mylog("execCommandLocally -- command: "+ command);

	String commandOutput=""; 
	try {
	    Process p = rt.exec(command);
	    BufferedReader in=
	      new BufferedReader(new InputStreamReader(p.getInputStream()));
	    
	    BufferedReader err= 
	      new BufferedReader(new InputStreamReader(p.getErrorStream()));

	    String stdoutAndStderr = "STDOUT:\n";
	    String line;
	    while((line=in.readLine())!= null) {
	      commandOutput+=line+"\n";
	      stdoutAndStderr+=line;
	    }
	    in.close();
	    stdoutAndStderr+="\nSTDERR:\n";
	    while ((line=err.readLine())!=null) {
	      stdoutAndStderr+=line;
	    }
	    err.close();
	    p.waitFor();
	    
	}
	catch (Exception eio) {
	  System.err.println(eio.getMessage());
	  eio.printStackTrace();
	}
	return commandOutput;
    }
  
  /**======================================================================
   * Execute the command remotely, using either rsh or ssh as specified
   * by the "carrier" variable. Returns the standard output of the command.
   * It is assumed that if ssh is used, it is set up so as not to need a
   * password or passphrase before executing.
   * The variable carrier either equals rsh or ssh.
   *======================================================================*/
  public String execRemoteCommand(String host,String user, 
				  String command, String carrier) {
      mylog("Executing remote command "+ command);
      Runtime rt = Runtime.getRuntime();
      String remoteCommand = carrier+" -l "+user+" "+host+" "+command;
      mylog("execCommandOnBackend -- command: "+ remoteCommand);
      
      //    if(!carrier.equals("rsh") || !carrier.equals("ssh")){
      // return "Improper communication method to remote system!";
      //}
      
      String commandOutput="";
      try {
	  Process p = rt.exec(remoteCommand);
	    BufferedReader in=
	      new BufferedReader(new InputStreamReader(p.getInputStream()));
	    
	    BufferedReader err=
	      new BufferedReader(new InputStreamReader(p.getErrorStream()));
	    
	    String stdoutAndStderr="STDOUT:\n";
	    String line;
	    while((line=in.readLine())!= null) {
	      commandOutput+=line+"\n";
	      stdoutAndStderr+=line;
	    }
	    in.close();
	    stdoutAndStderr+="\nSTDERR:\n";
	    while ((line=err.readLine())!=null) {
	      stdoutAndStderr+=line;
	    }
	    err.close();
	    p.waitFor();
	}
	catch (Exception eio) {
	  System.err.println(eio.getMessage());
	  eio.printStackTrace();
	}
	return commandOutput;
  }

  
  /**======================================================================
   * Copy a file from the backend to the server.
   *======================================================================*/
  public String copyFileFromBackend(String options,String user,String host,
				    String remoteFile,String localFile,
				    String carrier){
    
    
    if(!carrier.equals("rcp") || !carrier.equals("scp"))
      return "Improper communication method to remote system!";
    
    Runtime rt = Runtime.getRuntime();
    if(options.length() > 0) {
      String optionsTrim = options.trim();
      if(! optionsTrim.startsWith("-")) {
	options = "-" + optionsTrim;
      } 
    }
    
    String copyCommand = carrier+" "+options + " " + user.trim()
      + "@" + host.trim() +  ":" + remoteFile.trim() + " " +
      localFile;
    mylog("copyFileFromBackend  -- command: "+ copyCommand);
    String commandOutput="";
    try {
      Process p = rt.exec(copyCommand);
      BufferedReader in=
	new BufferedReader(new InputStreamReader(p.getInputStream()));
      BufferedReader err=
	new BufferedReader(new InputStreamReader(p.getErrorStream()));
      String stdoutAndStderr="STDOUT:\n";
      
      String line;
      while((line=in.readLine())!= null) {
	commandOutput+=line+"\n";
	stdoutAndStderr+=line;
      }
      in.close();
      stdoutAndStderr+="\nSTDERR:\n";
      while ((line=err.readLine())!=null) {
	stdoutAndStderr+=line;
      }
      err.close();
      p.waitFor();
    }
    catch (Exception eio) {
      System.err.println(eio.getMessage());
      eio.printStackTrace();
    }
    return commandOutput;
  }

  /**======================================================================
   * Copy a file from the server to the backend.
   *======================================================================*/
  public String copyFileToBackend(String options,String localFile,String user,
				  String host,String remoteFile,
				  String carrier){
    if(!carrier.equals("rcp") || !carrier.equals("scp"))
      return "Improper communication method to remote system!";
    
    Runtime rt = Runtime.getRuntime();
    if(options.length() > 0) {
      String optionsTrim = options.trim();
      if(! optionsTrim.startsWith("-")) {
	options = "-" + optionsTrim;
      } 
    }
    
    String copyCommand = carrier+" " + options + " " + localFile + " " +
      user.trim() + "@" + host.trim() +  ":" + remoteFile.trim();
    System.err.println("copyFileToBackend  -- command: "+ copyCommand);
    
    String commandOutput="";
    try {
      Process p = rt.exec(copyCommand);
      BufferedReader in=
	new BufferedReader(new InputStreamReader(p.getInputStream()));
      BufferedReader err=
	new BufferedReader(new InputStreamReader(p.getErrorStream()));
      String stdoutAndStderr="STDOUT:\n";
      
      String line;
      while((line=in.readLine())!= null) {
	  //Keep overriding commanOutput because we only want the
	  //last line of standard input.
	commandOutput=line+"\n";
	stdoutAndStderr+=line;
      }
      in.close();
      stdoutAndStderr+="\nSTDERR:\n";
      while ((line=err.readLine())!=null) {
	stdoutAndStderr+=line;
      }
      err.close();
      p.waitFor();
    }
    catch (Exception eio) {
      System.err.println(eio.getMessage());
      eio.printStackTrace();
    }
    
    return commandOutput;
  }
  

    /**
     * Print the argument to the log file.
     */
    private void mylog(String logline) {
        logFile.println(logline);
        logFile.flush();
    }
    
  /**
   * Return the time.
   */
  private String getTime() {
    DateFormat longDate=
      DateFormat.getDateTimeInstance(DateFormat.FULL,DateFormat.FULL);
    String time=longDate.format(new Date());
    return time;
  }
}
